@@ -207,17 +207,9 @@ void x64_classify_post_merge(ByteSize size, X64Class *lo_class, X64Class *hi_cla
 	// If X87UP is not before X87 => mem
 	// If size > 16 && first isn't SSE or any other is not SSEUP => mem
 	// If SSEUP is not preceded by SSE/SSEUP => convert to SSE.
-	if (*hi_class == CLASS_MEMORY) goto DEFAULT_TO_MEMORY;
-	if (size > 16 && (*lo_class != CLASS_SSE || *hi_class != CLASS_SSEUP)) goto DEFAULT_TO_MEMORY;
-	if (*hi_class == CLASS_SSEUP && *lo_class != CLASS_SSE && *lo_class != CLASS_SSEUP)
-	{
-		// This can happen for unions for example
-		*hi_class = CLASS_SSE;
-	}
-	return;
-
-	DEFAULT_TO_MEMORY:
-	*lo_class = CLASS_MEMORY;
+	if (*hi_class == CLASS_MEMORY) *lo_class = CLASS_MEMORY;
+	if (size > 16 && (*lo_class != CLASS_SSE || *hi_class != CLASS_SSEUP)) *lo_class = CLASS_MEMORY;
+	if (*hi_class == CLASS_SSEUP && *lo_class != CLASS_SSE) *hi_class = CLASS_SSE;
 }
 
 void x64_classify_struct_union(Type *type, ByteSize offset_base, X64Class *current, X64Class *lo_class, X64Class *hi_class, NamedArgument named_arg)
@@ -380,7 +372,7 @@ static void x64_classify(Type *type, ByteSize offset_base, X64Class *lo_class, X
 	{
 		case LOWERED_TYPES:
 		case TYPE_FUNC_RAW:
-			UNREACHABLE
+			UNREACHABLE_VOID
 		case TYPE_VOID:
 			*current = CLASS_NO_CLASS;
 			break;
@@ -491,7 +483,7 @@ bool x64_contains_float_at_offset(Type *type, unsigned offset)
 static Type *x64_get_fp_type_at_offset(Type *type, unsigned ir_offset)
 {
 	if (!ir_offset && type_is_float(type)) return type;
-	if (type->type_kind == TYPE_STRUCT)
+	if (type->type_kind == TYPE_STRUCT || type->type_kind == TYPE_UNION)
 	{
 		Decl *element = x64_get_member_at_offset(type->decl, ir_offset);
 		return x64_get_fp_type_at_offset(element->type, ir_offset - element->offset);
@@ -560,7 +552,7 @@ AbiType x64_get_int_type_at_offset(Type *type, unsigned offset, Type *source_typ
 		case LOWERED_TYPES:
 		case TYPE_VOID:
 		case TYPE_FUNC_RAW:
-			UNREACHABLE
+			UNREACHABLE_VOID
 		case TYPE_U64:
 		case TYPE_I64:
 		case TYPE_POINTER:
@@ -600,7 +592,7 @@ AbiType x64_get_int_type_at_offset(Type *type, unsigned offset, Type *source_typ
 			if (offset < 16) return abi_type_get(type_ulong);
 			break;
 		case TYPE_FLEXIBLE_ARRAY:
-			UNREACHABLE
+			UNREACHABLE_VOID
 		case TYPE_ARRAY:
 		{
 			Type *element = type->array.base;
