@@ -25,10 +25,10 @@ typedef enum
 
 typedef struct CValue
 {
-	CValueType kind;
+	Type *type;
 	VariableId optional;
 	VariableId var;
-	Type *type;
+	CValueType kind;
 } CValue;
 
 typedef struct
@@ -36,10 +36,10 @@ typedef struct
 	HTable gen_decl;
 	HTable gen_def;
 	FILE *file;
+	OptionalCatch catch;
 	int x;
 	int typename;
 	int id_gen;
-	OptionalCatch catch;
 } GenContext;
 
 #define PRINTF(x, ...) fprintf(c->file, x, ## __VA_ARGS__) /* NOLINT */
@@ -91,10 +91,11 @@ static const char *c_type_name(GenContext *c, Type *type)
 		case TYPE_ANYFAULT:
 		case TYPE_TYPEID:
 		case TYPE_INTERFACE:
-		case TYPE_DISTINCT:
-		case TYPE_FUNC_RAW:
 		case TYPE_TYPEDEF:
+		case TYPE_FUNC_RAW:
+		case TYPE_ALIAS:
 		case TYPE_ENUM:
+		case TYPE_CONST_ENUM:
 		case TYPE_UNTYPED_LIST:
 		case TYPE_INFERRED_ARRAY:
 		case TYPE_INFERRED_VECTOR:
@@ -143,10 +144,11 @@ static bool c_emit_type_decl(GenContext *c, Type *type)
 		case ALL_FLOATS:
 		case TYPE_POINTER:
 			return false;
-		case TYPE_DISTINCT:
-		case TYPE_FUNC_RAW:
 		case TYPE_TYPEDEF:
+		case TYPE_FUNC_RAW:
+		case TYPE_ALIAS:
 		case TYPE_ENUM:
+		case TYPE_CONST_ENUM:
 		case TYPE_UNTYPED_LIST:
 		case TYPE_INFERRED_ARRAY:
 		case TYPE_INFERRED_VECTOR:
@@ -255,7 +257,7 @@ static bool c_emit_type_decl(GenContext *c, Type *type)
 			if (prev) return false;
 			c_emit_type_decl(c, type->array.base);
 			int id = ++c->typename;
-			PRINTF("typedef struct { %s ptr[%u]; } __c3_array%d;\n", c_type_name(c, type->array.base), type->array.len, id);
+			PRINTF("typedef struct { %s ptr[%llu]; } __c3_array%d;\n", c_type_name(c, type->array.base), (unsigned long long)type->array.len, id);
 			scratch_buffer_clear();
 			scratch_buffer_printf(" __c3_array%d", id);
 			htable_set(&c->gen_decl, type, scratch_buffer_copy());
@@ -360,7 +362,7 @@ static void c_emit_const_expr(GenContext *c, CValue *value, Expr *expr)
 			for (ArraySize i = 0; i < expr->const_expr.bytes.len; i++)
 			{
 				char b = expr->const_expr.bytes.ptr[i];
-				if (b >= ' ' || b < 127)
+				if (b >= ' ' && b < 127)
 				{
 					PRINTF("%c", b);
 					continue;
@@ -385,7 +387,7 @@ static void c_emit_const_expr(GenContext *c, CValue *value, Expr *expr)
 			break;
 		case CONST_UNTYPED_LIST:
 		case CONST_MEMBER:
-			UNREACHABLE
+			UNREACHABLE_VOID
 	}
 	PRINT("/* CONST EXPR */\n");
 }
@@ -412,10 +414,11 @@ static void c_emit_expr(GenContext *c, CValue *value, Expr *expr)
 		case EXPR_MAKE_SLICE:
 		case EXPR_INT_TO_BOOL:
 		case EXPR_VECTOR_FROM_ARRAY:
+		case EXPR_TWO:
 			break;
 		case NON_RUNTIME_EXPR:
 		case UNRESOLVED_EXPRS:
-			UNREACHABLE
+			UNREACHABLE_VOID
 		case EXPR_ACCESS_RESOLVED:
 			break;
 		case EXPR_ASM:
@@ -466,6 +469,7 @@ static void c_emit_expr(GenContext *c, CValue *value, Expr *expr)
 		case EXPR_MACRO_BODY_EXPANSION:
 			break;
 		case EXPR_MEMBER_GET:
+		case EXPR_MEMBER_SET:
 			break;
 		case EXPR_NAMED_ARGUMENT:
 			break;
@@ -520,6 +524,7 @@ static void c_emit_jump_to_optional_exit(GenContext *c, int value)
 static int c_emit_load(GenContext *c, VariableId id)
 {
 	TODO
+	UNREACHABLE
 }
 
 static void c_value_fold_optional(GenContext *c, CValue *value)
@@ -578,7 +583,7 @@ static void c_emit_local_decl(GenContext *c, Decl *decl, CValue *value)
 		case VARDECL_GLOBAL:
 		case VARDECL_MEMBER:
 		case VARDECL_BITMEMBER:
-			UNREACHABLE;
+			UNREACHABLE_VOID;
 		case VARDECL_PARAM:
 		case VARDECL_UNWRAPPED:
 		case VARDECL_ERASE:
@@ -586,7 +591,7 @@ static void c_emit_local_decl(GenContext *c, Decl *decl, CValue *value)
 			return;
 		case VARDECL_LOCAL_CT:
 		case VARDECL_LOCAL_CT_TYPE:
-			UNREACHABLE
+			UNREACHABLE_VOID
 	}
 
 	// Get the declaration and the LLVM type.
@@ -633,9 +638,50 @@ static void c_emit_local_decl(GenContext *c, Decl *decl, CValue *value)
 		value->kind = CV_VALUE;
 	}
 
-	PRINT("/* TODO ZERO INIT */\n");
-	//llvm_store_zero(c, value);
-	//llvm_value_set(value, llvm_get_zero(c, var_type), var_type);
+	switch (var_type->type_kind)
+	{
+		case TYPE_BOOL:
+			PRINTF("___var_%d = false;\n", value->var);
+			break;
+		case ALL_INTS:
+		case ALL_FLOATS:
+			PRINTF("___var_%d = 0;\n", value->var);
+			break;
+		case TYPE_POISONED:
+		case TYPE_VOID:
+		case TYPE_TYPEDEF:
+		case TYPE_CONST_ENUM:
+		case TYPE_FUNC_RAW:
+		case TYPE_BITSTRUCT:
+		case TYPE_ALIAS:
+		case TYPE_UNTYPED_LIST:
+		case TYPE_FLEXIBLE_ARRAY:
+		case TYPE_INFERRED_ARRAY:
+		case TYPE_INFERRED_VECTOR:
+		case TYPE_OPTIONAL:
+		case TYPE_WILDCARD:
+		case TYPE_TYPEINFO:
+		case TYPE_MEMBER:
+			UNREACHABLE_VOID
+		case TYPE_ANY:
+		case TYPE_INTERFACE:
+			PRINTF("___var_%d = (c3_any_t){ NULL, NULL };\n", value->var);
+			break;
+		case TYPE_ANYFAULT:
+		case TYPE_TYPEID:
+		case TYPE_FUNC_PTR:
+		case TYPE_POINTER:
+		case TYPE_ENUM:
+			PRINTF("___var_%d = 0;\n", value->var);
+			break;
+		case TYPE_STRUCT:
+		case TYPE_UNION:
+		case TYPE_SLICE:
+		case TYPE_ARRAY:
+		case TYPE_VECTOR:
+			PRINT("/* TODO ZERO INIT */\n");
+
+	}
 }
 
 static void c_emit_return(GenContext *c, Ast *stmt)
@@ -740,7 +786,7 @@ static void c_emit_stmt(GenContext *c, Ast *stmt)
 	switch (stmt->ast_kind)
 	{
 		case AST_POISONED:
-			UNREACHABLE
+			UNREACHABLE_VOID
 		case AST_ASM_STMT:
 			break;
 		case AST_ASM_BLOCK_STMT:
@@ -754,9 +800,9 @@ static void c_emit_stmt(GenContext *c, Ast *stmt)
 		case AST_CASE_STMT:
 			break;
 		case AST_COMPOUND_STMT:
-			PRINT("  {\n");
+			PRINT("{\n");
 			c_emit_stmt_chain(c, stmt->compound_stmt.first_stmt);
-			PRINT("  }\n");
+			PRINT("}\n");
 			return;
 		case AST_CONTINUE_STMT:
 			break;
@@ -771,7 +817,8 @@ static void c_emit_stmt(GenContext *c, Ast *stmt)
 		case AST_CT_FOR_STMT:
 		case AST_CT_IF_STMT:
 		case AST_CT_SWITCH_STMT:
-			UNREACHABLE
+		case AST_CT_TYPE_ASSIGN_STMT:
+			UNREACHABLE_VOID
 		case AST_DECLARE_STMT:
 		{
 			CValue value;
@@ -1026,4 +1073,4 @@ void **c_gen(Module** modules, unsigned module_count)
 		c_gen_module(modules[i], i);
 	}
 	return NULL;
-}
\ No newline at end of file
+}
