@@ -156,26 +156,26 @@ typedef enum FLAG_ATTR
 
 typedef enum
 {
-	ASM_ARG_REG,
-	ASM_ARG_ADDR,
-	ASM_ARG_REGVAR,
-	ASM_ARG_ADDROF,
-	ASM_ARG_MEMVAR,
-	ASM_ARG_VALUE,
-	ASM_ARG_INT,
+	ASM_ARG_REG,     // A register
+	ASM_ARG_ADDR,    // [...]
+	ASM_ARG_REGVAR,  // foo - variable in a register, r/=r/+r
+	ASM_ARG_MEMADDR, // &foo - address to a variable
+	ASM_ARG_MEMVAR,  // [&foo]
+	ASM_ARG_VALUE,   // -1, (expr), 1, 3.0
+	ASM_ARG_INT,     // Converted from VALUE
 } AsmArgKind;
 
 typedef enum
 {
-	ASM_SCALE_1,
+	ASM_SCALE_1,    // *
 	ASM_SCALE_2,
 	ASM_SCALE_4,
 	ASM_SCALE_8,
-	ASM_SCALE_SHR,
-	ASM_SCALE_SHL,
-	ASM_SCALE_ASHL,
-	ASM_SCALE_ROR,
-	ASM_SCALE_RRX,
+	ASM_SCALE_SHR,  // >>
+	ASM_SCALE_SHL,  // <<
+	ASM_SCALE_ASHL, // >>>
+	ASM_SCALE_ROR,  // ror
+	ASM_SCALE_RRX,  // rrx
 } AsmOffsetType;
 
 typedef enum
@@ -208,6 +208,7 @@ typedef enum
 	AST_CT_FOR_STMT,
 	AST_CT_IF_STMT,
 	AST_CT_SWITCH_STMT,
+	AST_CT_TYPE_ASSIGN_STMT,
 	AST_DECLARE_STMT,
 	AST_DECLS_STMT,
 	AST_DEFAULT_STMT,
@@ -250,7 +251,7 @@ typedef enum FLAG_ATTR
 	ATTR_CALL = 1 << 12,
 	ATTR_BITSTRUCT = 1 << 13,
 	ATTR_MACRO = 1 << 14,
-	ATTR_DISTINCT = 1 << 15,
+	ATTR_TYPEDEF = 1 << 15,
 	ATTR_ENUM_VALUE = 1 << 16,
 	ATTR_INTERFACE_METHOD = 1 << 17,
 	ATTR_PARAM = 1 << 18,
@@ -259,11 +260,12 @@ typedef enum FLAG_ATTR
 
 #define EXPORTED_USER_DEFINED_TYPES (ATTR_ENUM | ATTR_UNION | ATTR_STRUCT | ATTR_FAULT)
 #define CALLABLE_TYPE (ATTR_FUNC | ATTR_INTERFACE_METHOD | ATTR_MACRO | ATTR_FNTYPE)
-#define USER_DEFINED_TYPES EXPORTED_USER_DEFINED_TYPES | ATTR_BITSTRUCT | ATTR_DISTINCT
+#define USER_DEFINED_TYPES (EXPORTED_USER_DEFINED_TYPES | ATTR_BITSTRUCT | ATTR_TYPEDEF)
 
 typedef enum
 {
 	ATTRIBUTE_ALIGN,
+	ATTRIBUTE_ALLOW_DEPRECATED,
 	ATTRIBUTE_BENCHMARK,
 	ATTRIBUTE_BIGENDIAN,
 	ATTRIBUTE_BUILTIN,
@@ -279,6 +281,7 @@ typedef enum
 	ATTRIBUTE_IF,
 	ATTRIBUTE_INLINE,
 	ATTRIBUTE_INIT,
+	ATTRIBUTE_JUMP,
 	ATTRIBUTE_LINK,
 	ATTRIBUTE_LITTLEENDIAN,
 	ATTRIBUTE_LOCAL,
@@ -295,6 +298,8 @@ typedef enum
 	ATTRIBUTE_NOSTRIP,
 	ATTRIBUTE_OBFUSCATE,
 	ATTRIBUTE_OPERATOR,
+	ATTRIBUTE_OPERATOR_R,
+	ATTRIBUTE_OPERATOR_S,
 	ATTRIBUTE_OPTIONAL,
 	ATTRIBUTE_OVERLAP,
 	ATTRIBUTE_PACKED,
@@ -302,8 +307,10 @@ typedef enum
 	ATTRIBUTE_PUBLIC,
 	ATTRIBUTE_PURE,
 	ATTRIBUTE_REFLECT,
+	ATTRIBUTE_SAFEINFER,
 	ATTRIBUTE_SAFEMACRO,
 	ATTRIBUTE_SECTION,
+	ATTRIBUTE_STRUCTLIKE,
 	ATTRIBUTE_TAG,
 	ATTRIBUTE_TEST,
 	ATTRIBUTE_UNUSED,
@@ -334,6 +341,7 @@ typedef enum
 	BINARYOP_CT_AND,
 	BINARYOP_CT_OR,
 	BINARYOP_CT_CONCAT,
+	BINARYOP_CT_CONCAT_ASSIGN,
 	// Don't change the ordering for GT to EQ or things will break
 	BINARYOP_GT,
 	BINARYOP_GE,
@@ -445,6 +453,7 @@ typedef enum
 	BUILTIN_EXP2,
 	BUILTIN_EXPECT,
 	BUILTIN_EXPECT_WITH_PROBABILITY,
+	BUILTIN_FENCE,
 	BUILTIN_FLOOR,
 	BUILTIN_FMA,
 	BUILTIN_FMULADD,
@@ -456,6 +465,8 @@ typedef enum
 	BUILTIN_LOG,
 	BUILTIN_LOG10,
 	BUILTIN_LOG2,
+	BUILTIN_MATRIX_MUL,
+	BUILTIN_MATRIX_TRANSPOSE,
 	BUILTIN_MASKED_LOAD,
 	BUILTIN_MASKED_STORE,
 	BUILTIN_MAX,
@@ -485,14 +496,17 @@ typedef enum
 	BUILTIN_REVERSE,
 	BUILTIN_RETURNADDRESS,
 	BUILTIN_RINT,
+	BUILTIN_RND,
 	BUILTIN_ROUND,
 	BUILTIN_ROUNDEVEN,
 	BUILTIN_SAT_ADD,
 	BUILTIN_SAT_SHL,
 	BUILTIN_SAT_SUB,
+	BUILTIN_SAT_MUL,
 	BUILTIN_SCATTER,
 	BUILTIN_SELECT,
 	BUILTIN_SET_ROUNDING_MODE,
+	BUILTIN_SPRINTF,
 	BUILTIN_STR_HASH,
 	BUILTIN_STR_UPPER,
 	BUILTIN_STR_LOWER,
@@ -626,6 +640,7 @@ typedef enum
 	CONV_ANY,
 	CONV_INTERFACE,
 	CONV_ENUM,
+	CONV_RAW_ENUM,
 	CONV_FUNC,
 	CONV_TYPEID,
 	CONV_ANYFAULT,
@@ -656,8 +671,9 @@ typedef enum
 	DECL_CT_EXEC,
 	DECL_CT_INCLUDE,
 	DECL_DECLARRAY,
-	DECL_DEFINE,
-	DECL_DISTINCT,
+	DECL_ALIAS,
+	DECL_ALIAS_PATH,
+	DECL_TYPEDEF,
 	DECL_ENUM,
 	DECL_ENUM_CONSTANT,
 	DECL_ERASED,
@@ -669,8 +685,9 @@ typedef enum
 	DECL_LABEL,
 	DECL_MACRO,
 	DECL_INTERFACE,
+	DECL_CONST_ENUM,
 	DECL_STRUCT,
-	DECL_TYPEDEF,
+	DECL_TYPE_ALIAS,
 	DECL_UNION,
 	DECL_VAR,
 } DeclKind;
@@ -719,6 +736,7 @@ typedef enum
 {
 	EXPR_ACCESS_RESOLVED,
 	EXPR_ACCESS_UNRESOLVED,
+	EXPR_ADDR_CONVERSION,
 	EXPR_ASM,
 	EXPR_BENCHMARK_HOOK,
 	EXPR_BINARY,
@@ -728,16 +746,15 @@ typedef enum
 	EXPR_BUILTIN_ACCESS,
 	EXPR_CALL,
 	EXPR_CAST,
-	EXPR_CATCH_UNRESOLVED,
 	EXPR_CATCH,
+	EXPR_CATCH_UNRESOLVED,
 	EXPR_COMPILER_CONST,
 	EXPR_COMPOUND_LITERAL,
 	EXPR_COND,
 	EXPR_CONST,
-	EXPR_TYPECALL,
 	EXPR_CT_ARG,
+	EXPR_CT_ASSIGNABLE,
 	EXPR_CT_CALL,
-	EXPR_CT_CASTABLE,
 	EXPR_CT_DEFINED,
 	EXPR_CT_EVAL,
 	EXPR_CT_IDENT,
@@ -749,46 +766,49 @@ typedef enum
 	EXPR_DESIGNATOR,
 	EXPR_DISCARD,
 	EXPR_EMBED,
-	EXPR_VECTOR_TO_ARRAY,
-	EXPR_SLICE_TO_VEC_ARRAY,
-	EXPR_SCALAR_TO_VECTOR,
+	EXPR_ENUM_FROM_ORD,
 	EXPR_EXPRESSION_LIST,
-	EXPR_FORCE_UNWRAP,
+	EXPR_EXT_TRUNC,
 	EXPR_FLOAT_TO_INT,
+	EXPR_FORCE_UNWRAP,
 	EXPR_GENERIC_IDENT,
 	EXPR_HASH_IDENT,
 	EXPR_IDENTIFIER,
 	EXPR_INITIALIZER_LIST,
+	EXPR_INT_TO_BOOL,
 	EXPR_INT_TO_FLOAT,
 	EXPR_INT_TO_PTR,
-	EXPR_PTR_TO_INT,
+	EXPR_IOTA_DECL,
 	EXPR_LAMBDA,
 	EXPR_LAST_FAULT,
+	EXPR_LENGTHOF,
 	EXPR_MACRO_BLOCK,
 	EXPR_MACRO_BODY,
 	EXPR_MACRO_BODY_EXPANSION,
 	EXPR_MAKE_ANY,
 	EXPR_MAKE_SLICE,
 	EXPR_MEMBER_GET,
+	EXPR_MEMBER_SET,
 	EXPR_NAMED_ARGUMENT,
 	EXPR_NOP,
 	EXPR_OPERATOR_CHARS,
 	EXPR_OPTIONAL,
-	EXPR_ENUM_FROM_ORD,
 	EXPR_OTHER_CONTEXT,
 	EXPR_POINTER_OFFSET,
-	EXPR_ADDR_CONVERSION,
 	EXPR_POISONED,
-	EXPR_PTR_ACCESS,
 	EXPR_POST_UNARY,
+	EXPR_PTR_ACCESS,
+	EXPR_PTR_TO_INT,
+	EXPR_RECAST,
 	EXPR_RETHROW,
 	EXPR_RETVAL,
 	EXPR_RVALUE,
-	EXPR_RECAST,
+	EXPR_SCALAR_TO_VECTOR,
 	EXPR_SLICE,
-	EXPR_SLICE_LEN,
 	EXPR_SLICE_ASSIGN,
 	EXPR_SLICE_COPY,
+	EXPR_SLICE_LEN,
+	EXPR_SLICE_TO_VEC_ARRAY,
 	EXPR_SPLAT,
 	EXPR_STRINGIFY,
 	EXPR_SUBSCRIPT,
@@ -800,16 +820,17 @@ typedef enum
 	EXPR_TRY,
 	EXPR_TRY_UNRESOLVED,
 	EXPR_TRY_UNWRAP_CHAIN,
+	EXPR_TWO,
+	EXPR_TYPECALL,
 	EXPR_TYPEID,
 	EXPR_TYPEID_INFO,
 	EXPR_TYPEINFO,
-	EXPR_UNRESOLVED_IDENTIFIER,
 	EXPR_UNARY,
+	EXPR_UNRESOLVED_IDENTIFIER,
 	EXPR_VASPLAT,
 	EXPR_VECTOR_FROM_ARRAY,
-	EXPR_EXT_TRUNC,
-	EXPR_INT_TO_BOOL,
-	EXPR_LAST = EXPR_VASPLAT
+	EXPR_VECTOR_TO_ARRAY,
+	EXPR_LAST = EXPR_VECTOR_TO_ARRAY
 } ExprKind;
 
 typedef enum
@@ -838,17 +859,18 @@ typedef enum
 	INTROSPECT_TYPE_ANYFAULT = 6,
 	INTROSPECT_TYPE_ANY = 7,
 	INTROSPECT_TYPE_ENUM = 8,
-	INTROSPECT_TYPE_STRUCT = 9,
-	INTROSPECT_TYPE_UNION = 10,
-	INTROSPECT_TYPE_BITSTRUCT = 11,
-	INTROSPECT_TYPE_FUNC = 12,
-	INTROSPECT_TYPE_OPTIONAL = 13,
-	INTROSPECT_TYPE_ARRAY = 14,
-	INTROSPECT_TYPE_SLICE = 15,
-	INTROSPECT_TYPE_VECTOR = 16,
-	INTROSPECT_TYPE_DISTINCT = 17,
-	INTROSPECT_TYPE_POINTER = 18,
-	INTROSPECT_TYPE_INTERFACE = 19,
+	INTROSPECT_TYPE_CONST_ENUM = 9,
+	INTROSPECT_TYPE_STRUCT = 10,
+	INTROSPECT_TYPE_UNION = 11,
+	INTROSPECT_TYPE_BITSTRUCT = 12,
+	INTROSPECT_TYPE_FUNC = 13,
+	INTROSPECT_TYPE_OPTIONAL = 14,
+	INTROSPECT_TYPE_ARRAY = 15,
+	INTROSPECT_TYPE_SLICE = 16,
+	INTROSPECT_TYPE_VECTOR = 17,
+	INTROSPECT_TYPE_DISTINCT = 18,
+	INTROSPECT_TYPE_POINTER = 19,
+	INTROSPECT_TYPE_INTERFACE = 20,
 } IntrospectType;
 
 typedef enum
@@ -896,14 +918,60 @@ typedef enum
 	OBJ_FORMAT_AOUT,
 } ObjectFormatType;
 
+typedef enum
+{
+	OVERLOAD_TYPE_LEFT = 1,
+	OVERLOAD_TYPE_RIGHT = 2,
+	OVERLOAD_TYPE_SYMMETRIC = OVERLOAD_TYPE_LEFT | OVERLOAD_TYPE_RIGHT,
+} OverloadType;
+
+typedef enum
+{
+	OVERLOAD_MATCH_ERROR = -1,
+	OVERLOAD_MATCH_NONE = 0,
+	OVERLOAD_MATCH_EXACT,
+	OVERLOAD_MATCH_CONVERSION,
+	OVERLOAD_MATCH_WILDCARD,
+	OVERLOAD_MATCH_AMBIGUOUS_WILDCARD,
+	OVERLOAD_MATCH_AMBIGUOUS_CONVERSION,
+	OVERLOAD_MATCH_AMBIGUOUS_EXACT,
+	OVERLOAD_MATCH_AMBIGUOUS_START = OVERLOAD_MATCH_AMBIGUOUS_WILDCARD
+} OverloadMatch;
 typedef enum
 {
 	OVERLOAD_ELEMENT_AT = 1,
 	OVERLOAD_ELEMENT_REF,
 	OVERLOAD_ELEMENT_SET,
 	OVERLOAD_LEN,
+	OVERLOAD_NEGATE,
+	OVERLOAD_UNARY_MINUS,
+	OVERLOAD_PLUS,
+	OVERLOAD_TYPED_START = OVERLOAD_PLUS,
+	OVERLOAD_MINUS,
+	OVERLOAD_MULTIPLY,
+	OVERLOAD_DIVIDE,
+	OVERLOAD_REMINDER,
+	OVERLOAD_AND,
+	OVERLOAD_OR,
+	OVERLOAD_XOR,
+	OVERLOAD_SHL,
+	OVERLOAD_SHR,
+	OVERLOAD_EQUAL,
+	OVERLOAD_NOT_EQUAL,
+	OVERLOAD_PLUS_ASSIGN,
+	OVERLOAD_MINUS_ASSIGN,
+	OVERLOAD_MULTIPLY_ASSIGN,
+	OVERLOAD_DIVIDE_ASSIGN,
+	OVERLOAD_REMINDER_ASSIGN,
+	OVERLOAD_AND_ASSIGN,
+	OVERLOAD_OR_ASSIGN,
+	OVERLOAD_XOR_ASSIGN,
+	OVERLOAD_SHL_ASSIGN,
+	OVERLOAD_SHR_ASSIGN,
+	OVERLOADS_COUNT = OVERLOAD_SHR_ASSIGN
 } OperatorOverload;
 
+#define OVERLOAD_NONE ((OperatorOverload)0)
 typedef enum
 {
 	OS_TYPE_UNKNOWN,
@@ -911,7 +979,7 @@ typedef enum
 	OS_TYPE_ANANAS,
 	OS_TYPE_CLOUD_ABI,
 	OS_TYPE_DRAGON_FLY,
-	OS_TYPE_FREE_BSD,
+	OS_TYPE_FREEBSD,
 	OS_TYPE_FUCHSIA,
 	OS_TYPE_IOS,
 	OS_TYPE_KFREEBSD,
@@ -1079,6 +1147,9 @@ typedef enum
 	TOKEN_CT_AND,           // &&&
 	TOKEN_CT_CONCAT,        // +++
 	TOKEN_CT_OR,            // |||
+	TOKEN_CT_TERNARY,       // ???
+	TOKEN_CT_CONCAT_ASSIGN, // +++=
+
 	// Literals.
 	TOKEN_IDENT,            // Any normal ident.
 	TOKEN_CONST_IDENT,      // Any purely uppercase ident,
@@ -1092,8 +1163,6 @@ typedef enum
 
 	// We want to parse #foo separately.
 	TOKEN_HASH_IDENT,       // #foobar
-	TOKEN_HASH_CONST_IDENT, // #FOOBAR
-	TOKEN_HASH_TYPE_IDENT,  // #Foobar
 
 	TOKEN_AT_IDENT,         // @macro
 	TOKEN_AT_CONST_IDENT,   // @MACRO
@@ -1136,6 +1205,7 @@ typedef enum
 
 	// Keywords
 	TOKEN_ALIAS,
+	TOKEN_FIRST_NON_TYPE_KEYWORD = TOKEN_ALIAS,
 	TOKEN_ASSERT,
 	TOKEN_ASM,
 	TOKEN_ATTRDEF,
@@ -1161,6 +1231,7 @@ typedef enum
 	TOKEN_IF,
 	TOKEN_INLINE,
 	TOKEN_IMPORT,
+	TOKEN_LENGTHOF,
 	TOKEN_MACRO,
 	TOKEN_MODULE,
 	TOKEN_NEXTCASE,
@@ -1202,6 +1273,7 @@ typedef enum
 	TOKEN_CT_IF,                // $if
 	TOKEN_CT_INCLUDE,           // $include
 	TOKEN_CT_IS_CONST,          // $is_const
+	TOKEN_CT_KINDOF,            // $kindof
 	TOKEN_CT_NAMEOF,            // $nameof
 	TOKEN_CT_OFFSETOF,          // $offsetof
 	TOKEN_CT_QNAMEOF,           // $qnameof
@@ -1217,11 +1289,11 @@ typedef enum
 	TOKEN_CT_VAEXPR,            // $vaexpr,
 	TOKEN_CT_VASPLAT,           // $vasplat,
 	TOKEN_LAST_KEYWORD = TOKEN_CT_VASPLAT,
-	TOKEN_DOCS_START,       // <*
-	TOKEN_DOCS_END,         // *>
+	TOKEN_DOCS_START,           // <*
+	TOKEN_DOCS_END,             // *>
 	TOKEN_DOCS_EOL,
 
-	TOKEN_EOF,              // \n - SHOULD ALWAYS BE THE LAST TOKEN.
+	TOKEN_EOF,                  // \n - SHOULD ALWAYS BE THE LAST TOKEN.
 
 	TOKEN_LAST = TOKEN_EOF,
 } TokenType;
@@ -1308,19 +1380,20 @@ typedef enum
 	TYPE_F128,
 	TYPE_FLOAT_LAST = TYPE_F128,
 	TYPE_NUM_LAST = TYPE_FLOAT_LAST,
-	TYPE_DISTINCT,
 	TYPE_ANY,
 	TYPE_INTERFACE,
 	TYPE_ANYFAULT,
 	TYPE_TYPEID,
 	TYPE_FUNC_PTR,
 	TYPE_POINTER,
+	TYPE_TYPEDEF,
+	TYPE_CONST_ENUM,
 	TYPE_ENUM,
 	TYPE_FUNC_RAW,
 	TYPE_STRUCT,
 	TYPE_UNION,
 	TYPE_BITSTRUCT,
-	TYPE_TYPEDEF,
+	TYPE_ALIAS,
 	TYPE_UNTYPED_LIST,
 	TYPE_SLICE,
 	TYPE_ARRAY,
@@ -1350,6 +1423,8 @@ typedef enum
 	TYPE_PROPERTY_IS_ORDERED,
 	TYPE_PROPERTY_IS_SUBSTRUCT,
 	TYPE_PROPERTY_LEN,
+	TYPE_PROPERTY_LOOKUP,
+	TYPE_PROPERTY_LOOKUP_FIELD,
 	TYPE_PROPERTY_MAX,
 	TYPE_PROPERTY_MEMBERSOF,
 	TYPE_PROPERTY_METHODSOF,
@@ -1365,6 +1440,7 @@ typedef enum
 	TYPE_PROPERTY_QNAMEOF,
 	TYPE_PROPERTY_RETURNS,
 	TYPE_PROPERTY_SIZEOF,
+	TYPE_PROPERTY_SET,
 	TYPE_PROPERTY_TAGOF,
 	TYPE_PROPERTY_HAS_TAGOF,
 	TYPE_PROPERTY_VALUES,
@@ -1586,20 +1662,20 @@ typedef enum
 #define CT_AST \
  AST_CT_ASSERT: case AST_CT_ECHO_STMT: case AST_CT_ELSE_STMT: \
  case AST_CT_FOREACH_STMT: case AST_CT_FOR_STMT: \
- case AST_CT_IF_STMT: case AST_CT_SWITCH_STMT
+ case AST_CT_IF_STMT: case AST_CT_SWITCH_STMT: case AST_CT_TYPE_ASSIGN_STMT
 
 // -- Decl helper macros
 #define NON_TYPE_DECLS DECL_IMPORT: case DECL_MACRO: \
 	case DECL_DECLARRAY: case DECL_ATTRIBUTE: case DECL_LABEL: \
-	case DECL_DEFINE: case DECL_CT_ASSERT: case DECL_CT_EXEC: case DECL_FAULT: \
+	case DECL_ALIAS: case DECL_CT_ASSERT: case DECL_CT_EXEC: case DECL_FAULT: \
 	case DECL_CT_ECHO: case DECL_CT_INCLUDE: case DECL_GROUP: \
 	case DECL_BODYPARAM: case DECL_VAR: case DECL_ENUM_CONSTANT: \
-	case DECL_POISONED
+	case DECL_POISONED: case DECL_ALIAS_PATH
 
 // -- Expr helper macros
 #define NON_RUNTIME_EXPR EXPR_POISONED: \
 		case EXPR_CT_DEFINED: \
-		case EXPR_CT_CASTABLE: case EXPR_CT_IS_CONST: \
+		case EXPR_CT_ASSIGNABLE: case EXPR_CT_IS_CONST: \
 		case EXPR_CT_ARG: case EXPR_TYPEINFO: case EXPR_CT_IDENT: case EXPR_HASH_IDENT: \
 		case EXPR_COMPILER_CONST: case EXPR_CT_CALL:  \
 		case EXPR_SPLAT: case EXPR_STRINGIFY: case EXPR_TYPECALL: \
@@ -1630,14 +1706,14 @@ static_assert(EXPR_LAST < 128, "Too many expression types");
 #define TYPELIKE_TOKENS TYPE_TOKENS: case TOKEN_TYPE_IDENT: case CT_TYPE_TOKENS
 
 // -- Types helper macros
-#define FLATTENED_TYPES TYPE_DISTINCT: case TYPE_OPTIONAL: case TYPE_TYPEDEF
+#define FLATTENED_TYPES TYPE_TYPEDEF: case TYPE_OPTIONAL: case TYPE_ALIAS
 #define ALL_SIGNED_INTS TYPE_I8: case TYPE_I16: case TYPE_I32: case TYPE_I64: case TYPE_I128
 #define ALL_UNSIGNED_INTS TYPE_U8: case TYPE_U16: case TYPE_U32: case TYPE_U64: case TYPE_U128
 #define ALL_FLOATS TYPE_BF16: case TYPE_F16: case TYPE_F32: case TYPE_F64: case TYPE_F128
 
-#define LOWERED_TYPES CT_TYPES: case TYPE_ENUM: case TYPE_TYPEDEF: case TYPE_TYPEID: \
-  case TYPE_DISTINCT: case TYPE_ANYFAULT: case TYPE_BITSTRUCT: \
-  case TYPE_OPTIONAL: case TYPE_INTERFACE
+#define LOWERED_TYPES CT_TYPES: case TYPE_ENUM: case TYPE_ALIAS: case TYPE_TYPEID: \
+  case TYPE_TYPEDEF: case TYPE_ANYFAULT: case TYPE_BITSTRUCT: \
+  case TYPE_OPTIONAL: case TYPE_INTERFACE: case TYPE_CONST_ENUM
 
 #define CT_TYPES TYPE_TYPEINFO: case TYPE_INFERRED_ARRAY: case TYPE_INFERRED_VECTOR: case TYPE_UNTYPED_LIST: \
 case TYPE_POISONED: case TYPE_MEMBER: case TYPE_WILDCARD
@@ -1648,4 +1724,5 @@ case TYPE_U8: case TYPE_U16: case TYPE_U32: case TYPE_U64: case TYPE_U128
 #define UNRESOLVED_EXPRS EXPR_TRY_UNRESOLVED: case EXPR_ACCESS_UNRESOLVED: \
 	case EXPR_CATCH_UNRESOLVED: case EXPR_UNRESOLVED_IDENTIFIER: case EXPR_CAST: \
 	case EXPR_TYPEID: case EXPR_EMBED: case EXPR_VASPLAT: case EXPR_OTHER_CONTEXT: \
+	case EXPR_IOTA_DECL: case EXPR_LENGTHOF: \
 	case EXPR_GENERIC_IDENT: case EXPR_COMPOUND_LITERAL: case EXPR_MACRO_BODY: case EXPR_CT_SUBSCRIPT
