@@ -10,7 +10,7 @@ static inline void llvm_emit_return_value(GenContext *context, LLVMValueRef valu
 static void llvm_expand_from_args(GenContext *c, Type *type, LLVMValueRef ref, unsigned *index, AlignSize alignment);
 static inline void llvm_process_parameter_value(GenContext *c, Decl *decl, ABIArgInfo *info, unsigned *index);
 static inline void llvm_emit_func_parameter(GenContext *context, Decl *decl, ABIArgInfo *abi_info, unsigned *index, unsigned real_index);
-static inline void llvm_emit_body(GenContext *c, LLVMValueRef function, FunctionPrototype *prototype, Signature *signature, Ast *body, Decl *decl);
+static inline void llvm_emit_body(GenContext *c, LLVMValueRef function, FunctionPrototype *prototype, Signature *signature, Ast *body, Decl *decl, bool is_naked);
 
 
 /**
@@ -55,11 +55,12 @@ bool llvm_emit_check_block_branch(GenContext *c)
 	return true;
 }
 
-void llvm_emit_br(GenContext *c, LLVMBasicBlockRef next_block)
+bool llvm_emit_br(GenContext *c, LLVMBasicBlockRef next_block)
 {
-	if (!llvm_emit_check_block_branch(c)) return;
+	if (!llvm_emit_check_block_branch(c)) return false;
 	c->current_block = NULL;
 	LLVMBuildBr(c->builder, next_block);
+	return true;
 }
 
 
@@ -334,7 +335,7 @@ void llvm_emit_return_abi(GenContext *c, BEValue *return_value, BEValue *optiona
 		case ABI_ARG_EXPAND:
 			// Expands to multiple slots -
 			// Not applicable to return values.
-			UNREACHABLE
+			UNREACHABLE_VOID
 		case ABI_ARG_EXPAND_COERCE:
 		{
 			// Pick the return as an address.
@@ -382,7 +383,7 @@ DIRECT_RETURN:
 			return;
 		}
 	}
-	UNREACHABLE
+	UNREACHABLE_VOID
 }
 
 void llvm_emit_return_implicit(GenContext *c)
@@ -431,13 +432,13 @@ void llvm_emit_function_body(GenContext *c, Decl *decl)
 	llvm_emit_body(c,
 	               decl->backend_ref,
 	               type_get_resolved_prototype(decl->type),
-	               decl->func_decl.attr_naked ? NULL : &decl->func_decl.signature,
-	               astptr(decl->func_decl.body), decl);
+	               &decl->func_decl.signature,
+	               astptr(decl->func_decl.body), decl, decl->func_decl.attr_naked);
 }
 
 
 void llvm_emit_body(GenContext *c, LLVMValueRef function, FunctionPrototype *prototype, Signature *signature, Ast *body,
-                    Decl *decl)
+                    Decl *decl, bool is_naked)
 {
 	ASSERT(prototype && function && body);
 	// Signature is NULL if the function is naked.
@@ -494,9 +495,9 @@ void llvm_emit_body(GenContext *c, LLVMValueRef function, FunctionPrototype *pro
 	}
 
 
-	if (signature)
+	// Generate LLVMValueRef's for all parameters, so we can use them as local vars in code
+	if (!is_naked)
 	{
-		// Generate LLVMValueRef's for all parameters, so we can use them as local vars in code
 		FOREACH_IDX(i, Decl *, param, signature->params)
 		{
 			llvm_emit_func_parameter(c, param, prototype->abi_args[i], &arg, i);
@@ -516,7 +517,14 @@ void llvm_emit_body(GenContext *c, LLVMValueRef function, FunctionPrototype *pro
 	// Insert a return (and defer) if needed.
 	if (c->current_block && !LLVMGetBasicBlockTerminator(c->current_block))
 	{
-		llvm_emit_return_implicit(c);
+		if (!is_naked)
+		{
+			llvm_emit_return_implicit(c);
+		}
+		else
+		{
+			llvm_emit_unreachable(c);
+		}
 	}
 
 	LLVMBasicBlockRef last_block = LLVMGetLastBasicBlock(c->cur_func.ref);
@@ -663,7 +671,7 @@ void llvm_emit_dynamic_functions(GenContext *c, Decl **funcs)
 
 void llvm_emit_function_decl(GenContext *c, Decl *decl)
 {
-	ASSERT(decl->decl_kind == DECL_FUNC);
+	ASSERT_SPAN(decl, decl->decl_kind == DECL_FUNC);
 	// Resolve function backend type for function.
 	decl_append_links_to_global(decl);
 	LLVMValueRef function = llvm_get_ref(c, decl);
