@@ -18,14 +18,16 @@ static inline LLVMTypeRef llvm_type_from_decl(GenContext *c, Decl *decl)
 		case NON_TYPE_DECLS:
 		case DECL_FNTYPE:
 		case DECL_INTERFACE:
-			UNREACHABLE
+			UNREACHABLE_VOID
 		case DECL_BITSTRUCT:
 			return llvm_get_type(c, decl->strukt.container_type->type);
 		case DECL_FUNC:
-			UNREACHABLE
-		case DECL_TYPEDEF:
+			UNREACHABLE_VOID
+		case DECL_TYPE_ALIAS:
 			return llvm_get_type(c, decl->type);
-		case DECL_DISTINCT:
+		case DECL_CONST_ENUM:
+			return llvm_get_type(c, decl->enums.type_info->type);
+		case DECL_TYPEDEF:
 			return llvm_get_type(c, decl->distinct->type);
 		case DECL_STRUCT:
 		{
@@ -98,8 +100,8 @@ static void param_expand(GenContext *context, LLVMTypeRef** params_ref, Type *ty
 {
 	switch (type->type_kind)
 	{
-		case TYPE_TYPEDEF:
-			UNREACHABLE
+		case TYPE_ALIAS:
+			UNREACHABLE_VOID
 		case TYPE_ARRAY:
 			for (ArraySize i = type->array.len; i > 0; i--)
 			{
@@ -142,7 +144,7 @@ static void param_expand(GenContext *context, LLVMTypeRef** params_ref, Type *ty
 			vec_add(*params_ref, llvm_get_type(context, type));
 			return;
 	}
-	UNREACHABLE
+	UNREACHABLE_VOID
 }
 
 static inline void add_func_type_param(GenContext *c, Type *param_type, ABIArgInfo *arg_info, LLVMTypeRef **params)
@@ -209,7 +211,7 @@ LLVMTypeRef llvm_update_prototype_abi(GenContext *c, FunctionPrototype *prototyp
 	switch (ret_arg_info->kind)
 	{
 		case ABI_ARG_EXPAND:
-			UNREACHABLE;
+			UNREACHABLE_VOID;
 		case ABI_ARG_INDIRECT:
 			vec_add(*params, c->ptr_type);
 			retval = llvm_get_type(c, type_void);
@@ -235,7 +237,7 @@ LLVMTypeRef llvm_update_prototype_abi(GenContext *c, FunctionPrototype *prototyp
 			retval = llvm_get_type(c, call_return_type);
 			break;
 		case ABI_ARG_DIRECT_SPLIT_STRUCT_I32:
-			UNREACHABLE
+			UNREACHABLE_VOID
 		case ABI_ARG_DIRECT_COERCE_INT:
 			retval = LLVMIntTypeInContext(c->context, type_size(call_return_type) * 8);
 			break;
@@ -309,7 +311,7 @@ LLVMTypeRef llvm_get_type(GenContext *c, Type *any_type)
 	{
 		case LOWERED_TYPES:
 			// If this is reachable, then we're not doing the proper lowering.
-			UNREACHABLE
+			UNREACHABLE_VOID
 		case TYPE_STRUCT:
 		case TYPE_UNION:
 			return any_type->backend_type = llvm_type_from_decl(c, any_type->decl);
@@ -544,7 +546,7 @@ static LLVMValueRef llvm_get_introspection_for_enum(GenContext *c, Type *type)
 		{
 			assert(values);
 			BEValue value;
-			llvm_emit_expr_global_value(c, &value, enum_vals[i]->enum_constant.args[ai]);
+			llvm_emit_expr_global_value(c, &value, enum_vals[i]->enum_constant.associated[ai]);
 			LLVMValueRef llvm_value = llvm_load_value_store(c, &value);
 			values[i] = llvm_value;
 			if (!val_type)
@@ -555,6 +557,7 @@ static LLVMValueRef llvm_get_introspection_for_enum(GenContext *c, Type *type)
 			if (val_type != LLVMTypeOf(llvm_value)) mixed = true;
 		}
 		Decl *associated_value = associated_values[ai];
+		if (!val_type)val_type = llvm_get_type(c, associated_value->type);
 		LLVMValueRef associated_value_arr = mixed ? llvm_get_packed_struct(c, values, elements)
 				: llvm_get_array(val_type, values, elements);
 		scratch_buffer_set_extern_decl_name(decl, true);
@@ -618,10 +621,12 @@ LLVMValueRef llvm_get_typeid(GenContext *c, Type *type)
 			return llvm_generate_introspection_global(c, NULL, type, INTROSPECT_TYPE_INTERFACE, NULL, 0, NULL, false);
 		case TYPE_POINTER:
 			return llvm_generate_introspection_global(c, NULL, type, INTROSPECT_TYPE_POINTER, type->pointer, 0, NULL, false);
-		case TYPE_DISTINCT:
-			return llvm_generate_introspection_global(c, NULL, type, INTROSPECT_TYPE_DISTINCT, type->decl->distinct->type, 0, NULL, false);
+		case TYPE_TYPEDEF:
+			return llvm_generate_introspection_global(c, NULL, type, INTROSPECT_TYPE_DISTINCT, type_inline(type), 0, NULL, false);
 		case TYPE_ENUM:
 			return llvm_get_introspection_for_enum(c, type);
+		case TYPE_CONST_ENUM:
+			return llvm_generate_introspection_global(c, NULL, type, INTROSPECT_TYPE_CONST_ENUM, type_inline(type), vec_size(type->decl->enums.values), NULL, false);
 		case TYPE_STRUCT:
 		case TYPE_UNION:
 			return llvm_get_introspection_for_struct_union(c, type);
@@ -640,10 +645,10 @@ LLVMValueRef llvm_get_typeid(GenContext *c, Type *type)
 			LLVMValueRef ref = llvm_generate_temp_introspection_global(c, type);
 			return llvm_generate_introspection_global(c, ref, type, INTROSPECT_TYPE_BITSTRUCT, type->decl->strukt.container_type->type, 0, NULL, false);
 		}
-		case TYPE_TYPEDEF:
+		case TYPE_ALIAS:
 			return llvm_get_typeid(c, type->canonical);
 		case CT_TYPES:
-			UNREACHABLE
+			UNREACHABLE_VOID
 		case TYPE_VOID:
 			return llvm_get_introspection_for_builtin_type(c, type, INTROSPECT_TYPE_VOID, 0);
 		case TYPE_BOOL:
@@ -667,4 +672,4 @@ LLVMValueRef llvm_get_typeid(GenContext *c, Type *type)
 			return llvm_get_introspection_for_builtin_type(c, type, INTROSPECT_TYPE_TYPEID, 0);
 	}
 	UNREACHABLE
-}
\ No newline at end of file
+}
