@@ -4,19 +4,19 @@
 
 #include "sema_internal.h"
 
-static inline bool sema_expr_analyse_struct_plain_initializer(SemaContext *context, Decl *assigned, Expr *initializer);
+static inline bool sema_expr_analyse_struct_plain_initializer(SemaContext *context, Decl *assigned, Expr *initializer, bool *no_match_ref);
 static inline bool sema_expr_analyse_array_plain_initializer(SemaContext *context, Type *assigned, Type *flattened,
-															 Expr *initializer);
-static inline bool sema_expr_analyse_untyped_initializer(SemaContext *context, Expr *initializer);
+                                                             Expr *initializer, bool *no_match_ref);
+static inline bool sema_expr_analyse_untyped_initializer(SemaContext *context, Expr *initializer, bool *no_match_ref);
 static bool sema_expr_analyse_designated_initializer(SemaContext *context, Type *assigned, Type *flattened,
-													 Expr *initializer);
+                                                     Expr *initializer, bool *no_match_ref);
 static inline void sema_not_enough_elements_error(SemaContext *context, Expr *initializer, int element);
-static inline bool sema_expr_analyse_initializer(SemaContext *context, Type *assigned_type, Type *flattened, Expr *expr);
+static inline bool sema_expr_analyse_initializer(SemaContext *context, Type *assigned_type, Type *flattened, Expr *expr, bool *no_match_ref);
 static void sema_create_const_initializer_from_designated_init(ConstInitializer *const_init, Expr *initializer);
 static Decl *sema_resolve_element_for_name(SemaContext *context, Decl **decls, DesignatorElement ***elements_ref, unsigned *index, bool is_substruct);
 static Type *sema_expr_analyse_designator(SemaContext *context, Type *current, Expr *expr, ArrayIndex *max_index, Decl **member_ptr);
 INLINE bool sema_initializer_list_is_empty(Expr *value);
-static Type *sema_find_type_of_element(SemaContext *context, Type *type, DesignatorElement ***elements_ref, unsigned *curr_index, bool *is_constant, bool *did_report_error, ArrayIndex *max_index, Decl **member_ptr);
+static Type *sema_find_type_of_element(SemaContext *context, Type *type, DesignatorElement ***elements_ref, unsigned *curr_index, bool *did_report_error, ArrayIndex *max_index, Decl **member_ptr);
 static ArrayIndex sema_analyse_designator_index(SemaContext *context, Expr *index);
 static void sema_update_const_initializer_with_designator(ConstInitializer *const_init,
 														  DesignatorElement **curr,
@@ -34,11 +34,7 @@ static inline void sema_update_const_initializer_with_designator_array(ConstInit
 																	   DesignatorElement **curr,
 																	   DesignatorElement **end,
 																	   Expr *value);
-static inline void sema_update_const_initializer_with_designator(
-		ConstInitializer *const_init,
-		DesignatorElement **curr,
-		DesignatorElement **end,
-		Expr *value);
+
 
 bool const_init_local_init_may_be_global_inner(ConstInitializer *init, bool top)
 {
@@ -186,7 +182,7 @@ static inline void sema_not_enough_elements_error(SemaContext *context, Expr *in
  * Perform analysis for a plain initializer, that is one initializing all fields.
  * @return true if analysis succeeds.
  */
-static inline bool sema_expr_analyse_struct_plain_initializer(SemaContext *context, Decl *assigned, Expr *initializer)
+static inline bool sema_expr_analyse_struct_plain_initializer(SemaContext *context, Decl *assigned, Expr *initializer, bool *no_match_ref)
 {
 	ASSERT(assigned->resolve_status == RESOLVE_DONE);
 	Expr **elements = initializer->initializer_list;
@@ -202,6 +198,7 @@ static inline bool sema_expr_analyse_struct_plain_initializer(SemaContext *conte
 	// 2. We don't support this actually, but we used to. Maybe we will in the future.
 	if (elements_needed == 0)
 	{
+		if (no_match_ref) goto NO_MATCH;
 		// Generate a nice error message for zero.
 		RETURN_SEMA_ERROR(elements[0], "Too many elements in initializer, it must be empty.");
 	}
@@ -218,7 +215,7 @@ static inline bool sema_expr_analyse_struct_plain_initializer(SemaContext *conte
 	}
 
 	// 3. Loop through all elements.
-	ArrayIndex max_loop = size > elements_needed ? size : elements_needed;
+	ArrayIndex max_loop = size > elements_needed ? size : (ArrayIndex)elements_needed;
 	for (ArrayIndex i = 0; i < max_loop; i++)
 	{
 		// 4. Check if we exceeded the list of elements in the struct/union.
@@ -226,6 +223,7 @@ static inline bool sema_expr_analyse_struct_plain_initializer(SemaContext *conte
 		//    user pinpoint where they put the double elements.
 		if (i >= elements_needed)
 		{
+			if (no_match_ref) goto NO_MATCH;
 			ASSERT(i < size);
 			RETURN_SEMA_ERROR(elements[i], "Too many elements in initializer, expected only %d.", elements_needed);
 		}
@@ -249,6 +247,7 @@ static inline bool sema_expr_analyse_struct_plain_initializer(SemaContext *conte
 			}
 			if (i >= size)
 			{
+				if (no_match_ref) goto NO_MATCH;
 				sema_not_enough_elements_error(context, initializer, (int)i);
 				return false;
 			}
@@ -261,7 +260,7 @@ static inline bool sema_expr_analyse_struct_plain_initializer(SemaContext *conte
 			int reduce_by = max_index_to_copy - i - 1;
 			size -= reduce_by;
 			elements_needed -= reduce_by;
-			max_loop = size > elements_needed ? size : elements_needed;
+			max_loop = size > elements_needed ? size : (ArrayIndex)elements_needed;
 			ASSERT(size <= vec_size(initializer->initializer_list));
 			vec_resize(initializer->initializer_list, (unsigned)size);
 			elements = initializer->initializer_list;
@@ -269,12 +268,13 @@ static inline bool sema_expr_analyse_struct_plain_initializer(SemaContext *conte
 		}
 		if (i >= size)
 		{
+			if (no_match_ref) goto NO_MATCH;
 			sema_not_enough_elements_error(context, initializer, i);
 			return false;
 		}
 		Expr *element = elements[i];
 		// 6. We know the required type, so resolve the expression.
-		if (!sema_analyse_expr_rhs(context, members[i]->type, element, true, NULL, false)) return false;
+		if (!sema_analyse_expr_rhs(context, members[i]->type, element, true, no_match_ref, false)) return false;
 		if (member->decl_kind == DECL_VAR && member->var.kind == VARDECL_BITMEMBER)
 		{
 			if (!sema_bit_assignment_check(context, element, members[i])) return false;
@@ -306,7 +306,9 @@ static inline bool sema_expr_analyse_struct_plain_initializer(SemaContext *conte
 
 	// 7. Done!
 	return true;
-
+NO_MATCH:;
+	*no_match_ref = true;
+	return false;
 }
 
 Expr *sema_create_struct_from_expressions(Decl *struct_decl, SourceSpan span, Expr **exprs)
@@ -320,7 +322,7 @@ Expr *sema_create_struct_from_expressions(Decl *struct_decl, SourceSpan span, Ex
  * @return true if analysis succeeds.
  */
 static inline bool sema_expr_analyse_array_plain_initializer(SemaContext *context, Type *assigned, Type *flattened,
-															 Expr *initializer)
+                                                             Expr *initializer, bool *no_match_ref)
 {
 	Expr **elements = initializer->initializer_list;
 	bool inferred_len = type_len_is_inferred(flattened);
@@ -328,9 +330,9 @@ static inline bool sema_expr_analyse_array_plain_initializer(SemaContext *contex
 	// We have the case where "Foo = int[*]"
 	if (inferred_len && !type_len_is_inferred(assigned))
 	{
-		ASSERT(assigned->type_kind == TYPE_TYPEDEF);
-		ASSERT(assigned->decl->decl_kind == DECL_TYPEDEF);
-		while (assigned->type_kind == TYPE_TYPEDEF) assigned = assigned->decl->type;
+		ASSERT(assigned->type_kind == TYPE_ALIAS);
+		ASSERT(assigned->decl->decl_kind == DECL_TYPE_ALIAS);
+		while (assigned->type_kind == TYPE_ALIAS) assigned = assigned->decl->type;
 		ASSERT(type_len_is_inferred(assigned));
 	}
 	// Prefer the typedef index: define Bar = int; Bar[1] => Bar and not int
@@ -342,9 +344,9 @@ static inline bool sema_expr_analyse_array_plain_initializer(SemaContext *contex
 
 	if (expected_members == 0 && !inferred_len)
 	{
+		if (no_match_ref) goto NO_MATCH;
 		// Generate a nice error message for zero.
-		SEMA_ERROR(elements[0], "Too many elements in initializer, it must be empty.");
-		return false;
+		RETURN_SEMA_ERROR(elements[0], "Too many elements in initializer, it must be empty.");
 	}
 
 	bool optional = false;
@@ -357,12 +359,12 @@ static inline bool sema_expr_analyse_array_plain_initializer(SemaContext *contex
 		Expr *element = elements[i];
 		if (!inferred_len && i >= expected_members)
 		{
-			SEMA_ERROR(element, "Too many elements in initializer, expected only %d.", expected_members);
-			return false;
+			if (no_match_ref) goto NO_MATCH;
+			RETURN_SEMA_ERROR(element, "Too many elements in initializer, expected only %d.", expected_members);
 		}
 		if (is_vector)
 		{
-			if (!sema_analyse_inferred_expr(context, inner_type, element)) return false;
+			if (!sema_analyse_inferred_expr(context, inner_type, element, no_match_ref)) return false;
 			Type *element_type = element->type;
 			Type *element_flat = type_flatten(element_type);
 			if (element_flat->type_kind == TYPE_VECTOR
@@ -371,19 +373,18 @@ static inline bool sema_expr_analyse_array_plain_initializer(SemaContext *contex
 				unsigned len = element_flat->array.len;
 				if (!inferred_len && i + len > expected_members)
 				{
-					SEMA_ERROR(element, "Too many elements in initializer when expanding, expected only %d.", expected_members);
-					return false;
+					RETURN_SEMA_ERROR(element, "Too many elements in initializer when expanding, expected only %d.", expected_members);
 				}
-				Expr *expr_list = expr_new_expr(EXPR_EXPRESSION_LIST, element);
+				Expr *expr_two = expr_new_expr(EXPR_TWO, element);
 				Decl *decl = decl_new_generated_var(element_type, VARDECL_LOCAL, element->span);
 				Expr *decl_expr = expr_generate_decl(decl, element);
-				vec_add(expr_list->expression_list, decl_expr);
+				expr_two->two_expr.first = decl_expr;
 				Expr *sub = expr_new_expr(EXPR_SUBSCRIPT, element);
 				sub->subscript_expr.expr = exprid(expr_variable(decl));
 				sub->subscript_expr.index.expr = exprid(expr_new_const_int(element->span, type_usz, 0));
-				vec_add(expr_list->expression_list, sub);
-				if (!sema_analyse_expr_rhs(context, inner_type, expr_list, true, NULL, false)) return false;
-				elements[i] = expr_list;
+				expr_two->two_expr.last = sub;
+				if (!sema_analyse_expr_rhs(context, inner_type, expr_two, true, NULL, false)) return false;
+				elements[i] = expr_two;
 				for (unsigned j = 1; j < len; j++)
 				{
 					sub = expr_new_expr(EXPR_SUBSCRIPT, element);
@@ -398,19 +399,19 @@ static inline bool sema_expr_analyse_array_plain_initializer(SemaContext *contex
 				optional = optional || IS_OPTIONAL(element);
 				continue;
 			}
-			if (!cast_implicit(context, element, inner_type, false)) return false;
+			if (!cast_implicit_checked(context, element, inner_type, false, no_match_ref)) return false;
 			optional = optional || IS_OPTIONAL(element);
 		}
 		else
 		{
-			if (!sema_analyse_expr_rhs(context, inner_type, element, true, NULL, false)) return false;
+			if (!sema_analyse_expr_rhs(context, inner_type, element, true, no_match_ref, false)) return false;
 			if (inner_is_inferred)
 			{
 				if (inferred_element)
 				{
-					if (!cast_implicit(context, element, inferred_element, false))
+					if (!cast_implicit_checked(context, element, inferred_element, false, no_match_ref))
 					{
-						SEMA_NOTE(elements[0], "Type inferred from here.");
+						if (!no_match_ref) SEMA_NOTE(elements[0], "Type inferred from here.");
 						return false;
 					}
 				}
@@ -426,8 +427,8 @@ static inline bool sema_expr_analyse_array_plain_initializer(SemaContext *contex
 	{
 		if (!inferred_element)
 		{
-			SEMA_ERROR(initializer, "Zero sized elements are not allowed when inferring size.");
-			return false;
+			if (no_match_ref) goto NO_MATCH;
+			RETURN_SEMA_ERROR(initializer, "Zero sized elements are not allowed when inferring size.");
 		}
 		inner_type = inferred_element;
 	}
@@ -445,8 +446,8 @@ static inline bool sema_expr_analyse_array_plain_initializer(SemaContext *contex
 
 	if (!inferred_len && expected_members > count)
 	{
-		SEMA_ERROR(elements[count - 1], "Too few elements in initializer, %d elements are needed.", expected_members);
-		return false;
+		if (no_match_ref) goto NO_MATCH;
+		RETURN_SEMA_ERROR(elements[count - 1], "Too few elements in initializer, %d elements are needed.", expected_members);
 	}
 
 	initializer->resolve_status = RESOLVE_DONE;
@@ -468,16 +469,24 @@ static inline bool sema_expr_analyse_array_plain_initializer(SemaContext *contex
 
 	// 7. Done!
 	return true;
+NO_MATCH:;
+	*no_match_ref = true;
+	return false;
 }
 
-static inline bool sema_expr_analyse_untyped_initializer(SemaContext *context, Expr *initializer)
+static inline bool sema_expr_analyse_untyped_initializer(SemaContext *context, Expr *initializer, bool *no_match_ref)
 {
 	Expr **init_list = initializer->initializer_list;
 	FOREACH(Expr *, element, init_list)
 	{
-		if (!sema_analyse_expr(context, element)) return false;
+		if (!sema_analyse_expr_rvalue(context, element)) return false;
 		if (!sema_cast_const(element))
 		{
+			if (no_match_ref)
+			{
+				*no_match_ref = true;
+				return false;
+			}
 			RETURN_SEMA_ERROR(element, "An untyped list can only have "
 									   "constant elements, you can try "
 									   "to type the list by prefixing the type and possibly enclosing it in parentheses, "
@@ -492,7 +501,7 @@ static inline bool sema_expr_analyse_untyped_initializer(SemaContext *context, E
 }
 
 static bool sema_expr_analyse_designated_initializer(SemaContext *context, Type *assigned, Type *flattened,
-													 Expr *initializer)
+                                                     Expr *initializer, bool *no_match_ref)
 {
 	Expr **init_expressions = initializer->designated_init_list;
 	Type *original = flattened->canonical;
@@ -517,7 +526,7 @@ static bool sema_expr_analyse_designated_initializer(SemaContext *context, Type
 			bitmember_count_without_value += 1;
 		}
 		if (!value) RETURN_SEMA_ERROR(expr, "This initializer needs a value.");
-		if (!sema_analyse_expr_rhs(context, result, value, true, NULL, false)) return false;
+		if (!sema_analyse_expr_rhs(context, result, value, true, no_match_ref, false)) return false;
 		if (is_bitmember)
 		{
 			if (!sema_bit_assignment_check(context, value, member)) return false;
@@ -529,13 +538,21 @@ static bool sema_expr_analyse_designated_initializer(SemaContext *context, Type
 			inner_type = type_no_optional(value->type);
 		}
 	}
-	if (bitmember_count_without_value != 0 && bitmember_count_without_value != vec_size(init_expressions)) {
+	if (bitmember_count_without_value != 0 && bitmember_count_without_value != vec_size(init_expressions))
+	{
 		RETURN_SEMA_ERROR(initializer, "Mixing the omission of initializers is not permitted.");
 	}
 	Type *type;
 	if (!is_structlike && is_inferred)
 	{
-		 type = type_from_inferred(flattened, type_get_indexed_type(assigned), (ArraySize)(max_index + 1));
+		if (type_is_infer_type(flattened))
+		{
+			type = type_from_inferred(flattened, inner_type, (ArraySize)(max_index + 1));
+		}
+		else
+		{
+			type = type_from_inferred(flattened, inner_type, flattened->array.len);
+		}
 	}
 	else
 	{
@@ -553,7 +570,7 @@ static bool sema_expr_analyse_designated_initializer(SemaContext *context, Type
 }
 
 
-static inline bool sema_expr_analyse_initializer(SemaContext *context, Type *assigned_type, Type *flattened, Expr *expr)
+static inline bool sema_expr_analyse_initializer(SemaContext *context, Type *assigned_type, Type *flattened, Expr *expr, bool *no_match_ref)
 {
 	// Note at this point this we either have
 	// EXPR_DESIGNATED_INITIALIZER_LIST
@@ -563,13 +580,13 @@ static inline bool sema_expr_analyse_initializer(SemaContext *context, Type *ass
 	// 1. Designated initializer is separately evaluated.
 	if (expr->expr_kind == EXPR_DESIGNATED_INITIALIZER_LIST)
 	{
-		return sema_expr_analyse_designated_initializer(context, assigned_type, flattened, expr);
+		return sema_expr_analyse_designated_initializer(context, assigned_type, flattened, expr,NULL);
 	}
 
 	if (expr->expr_kind == EXPR_CONST)
 	{
 		ASSERT(expr->const_expr.const_kind == CONST_INITIALIZER);
-		return cast_implicit(context, expr, assigned_type, false);
+		return cast_implicit_checked(context, expr, assigned_type, false, no_match_ref);
 	}
 	ASSERT(expr->expr_kind == EXPR_INITIALIZER_LIST);
 
@@ -587,8 +604,8 @@ static inline bool sema_expr_analyse_initializer(SemaContext *context, Type *ass
 	{
 		if (type_len_is_inferred(assigned_type))
 		{
-			SEMA_ERROR(expr, "Zero length arrays / vectors are not permitted.");
-			return false;
+			if (no_match_ref) goto NO_MATCH;
+			RETURN_SEMA_ERROR(expr, "Zero length arrays / vectors are not permitted.");
 		}
 		if (flattened == type_untypedlist)
 		{
@@ -602,7 +619,7 @@ static inline bool sema_expr_analyse_initializer(SemaContext *context, Type *ass
 	// 4. We might have a complist, because were analyzing $foo = { ... } or similar.
 	if (assigned_type == type_untypedlist)
 	{
-		return sema_expr_analyse_untyped_initializer(context, expr);
+		return sema_expr_analyse_untyped_initializer(context, expr, no_match_ref);
 	}
 
 	// 5. If not, then we see if we have an array.
@@ -610,14 +627,18 @@ static inline bool sema_expr_analyse_initializer(SemaContext *context, Type *ass
 		flattened->type_kind == TYPE_ARRAY ||
 		flattened->type_kind == TYPE_INFERRED_ARRAY ||
 		flattened->type_kind == TYPE_INFERRED_VECTOR ||
+		flattened->type_kind == TYPE_FLEXIBLE_ARRAY ||
 		flattened->type_kind == TYPE_SLICE ||
 		flattened->type_kind == TYPE_VECTOR)
 	{
-		return sema_expr_analyse_array_plain_initializer(context, assigned_type, flattened, expr);
+		return sema_expr_analyse_array_plain_initializer(context, assigned_type, flattened, expr, no_match_ref);
 	}
 
 	expr->type = assigned_type;
-	return sema_expr_analyse_struct_plain_initializer(context, flattened->decl, expr);
+	return sema_expr_analyse_struct_plain_initializer(context, flattened->decl, expr, no_match_ref);
+NO_MATCH:;
+	*no_match_ref = true;
+	return false;
 }
 
 /**
@@ -771,13 +792,13 @@ ConstInitializer *sema_merge_bitstruct_const_initializers(ConstInitializer *lhs,
 				lhs_expr->const_expr.ixx = int_or(lhs_expr->const_expr.ixx, rhs_expr->const_expr.ixx);
 				break;
 			default:
-				UNREACHABLE;
+				UNREACHABLE
 		}
 	}
 	return lhs;
 }
 
-bool sema_expr_analyse_initializer_list(SemaContext *context, Type *to, Expr *expr)
+bool sema_expr_analyse_initializer_list(SemaContext *context, Type *to, Expr *expr, bool *no_match_ref)
 {
 
 	if (!to) to = type_untypedlist;
@@ -804,8 +825,9 @@ bool sema_expr_analyse_initializer_list(SemaContext *context, Type *to, Expr *ex
 		case TYPE_BITSTRUCT:
 		case TYPE_INFERRED_ARRAY:
 		case TYPE_INFERRED_VECTOR:
+		case TYPE_FLEXIBLE_ARRAY:
 		case TYPE_VECTOR:
-			return sema_expr_analyse_initializer(context, to, flattened, expr);
+			return sema_expr_analyse_initializer(context, to, flattened, expr, no_match_ref);
 		case TYPE_SLICE:
 		{
 			if (is_zero_init)
@@ -815,7 +837,7 @@ bool sema_expr_analyse_initializer_list(SemaContext *context, Type *to, Expr *ex
 			}
 			// Resolve this as an inferred array.
 			Type *type = type_get_inferred_array(flattened->array.base);
-			if (!sema_expr_analyse_initializer(context, type, type, expr)) return false;
+			if (!sema_expr_analyse_initializer(context, type, type, expr, no_match_ref)) return false;
 			if (expr_is_const_initializer(expr))
 			{
 				ConstInitializer *init = expr->const_expr.initializer;
@@ -826,7 +848,12 @@ bool sema_expr_analyse_initializer_list(SemaContext *context, Type *to, Expr *ex
 			}
 			expr->resolve_status = RESOLVE_DONE;
 			expr_insert_addr(expr);
-			if (!sema_analyse_expr(context, expr)) return false;
+			if (!sema_analyse_expr_rvalue(context, expr)) return false;
+			if (no_match_ref)
+			{
+				if (!cast_explicit_silent(context, expr, to)) goto NO_MATCH;
+				return true;
+			}
 			return cast_explicit(context, expr, to);
 		}
 		case TYPE_POINTER:
@@ -836,14 +863,16 @@ bool sema_expr_analyse_initializer_list(SemaContext *context, Type *to, Expr *ex
 				expr_rewrite_to_const_zero(expr, to);
 				return true;
 			}
+			if (no_match_ref) goto NO_MATCH;
 			RETURN_SEMA_ERROR(expr, "Pointers cannot be initialized using an initializer list, instead you need to take the address of an array.");
 		case TYPE_VOID:
 		case TYPE_POISONED:
 		case TYPE_FUNC_RAW:
-		case TYPE_TYPEDEF:
+		case TYPE_ALIAS:
 		case TYPE_OPTIONAL:
 		case TYPE_TYPEINFO:
 		case TYPE_MEMBER:
+			if (no_match_ref) goto NO_MATCH;
 			RETURN_SEMA_ERROR(expr, "You cannot use %s with an initializer list.", type_quoted_error_string(to));
 		default:
 			if (is_zero_init)
@@ -854,7 +883,11 @@ bool sema_expr_analyse_initializer_list(SemaContext *context, Type *to, Expr *ex
 			}
 			break;
 	}
+	if (no_match_ref) goto NO_MATCH;
 	RETURN_SEMA_ERROR(expr, "You cannot use %s with a non-empty initializer list.", type_quoted_error_string(to));
+NO_MATCH:
+	*no_match_ref = true;
+	return false;
 }
 
 void const_init_rewrite_to_value(ConstInitializer *const_init, Expr *value)
@@ -943,18 +976,16 @@ static inline void sema_update_const_initializer_with_designator_struct(ConstIni
 	DesignatorElement **next_element = curr + 1;
 	bool is_last_path_element = next_element == end;
 
-	// Optimize in case this is a zero, e.g. [12].b = {}
-	if (is_last_path_element && sema_initializer_list_is_empty(value))
-	{
-		const_init->kind = CONST_INIT_ZERO;
-		ASSERT(type_flatten(value->type)->type_kind != TYPE_SLICE);
-		return;
-	}
 	Decl **elements = const_init->type->decl->strukt.members;
 
 	// Convert a zero struct and expand it into all its parts.
 	if (const_init->kind == CONST_INIT_ZERO)
 	{
+		if (is_last_path_element && sema_initializer_list_is_empty(value))
+		{
+			// In this case we can ignore it.
+			return;
+		}
 		const_init->init_struct = NULL;
 		// Allocate array containing all elements { a, b, c ... }
 		FOREACH_IDX(i, Decl *, el, elements)
@@ -1049,7 +1080,7 @@ static inline ConstInitializer *sema_update_const_initializer_at_index(ConstInit
 {
 	ConstInitializer **array_elements = const_init->init_array.elements;
 	ArrayIndex insert_index = *insert_index_ref;
-	ArrayIndex array_count = vec_size(array_elements);
+	ArrayIndex array_count = (ArrayIndex)vec_size(array_elements);
 	// Walk to the insert point or until we reached the end of the array.
 	while (insert_index < array_count && array_elements[insert_index]->init_array_value.index < index)
 	{
@@ -1180,7 +1211,7 @@ static inline void sema_update_const_initializer_with_designator(
 			sema_update_const_initializer_with_designator_array(const_init, curr, end, value);
 			return;
 		default:
-			UNREACHABLE
+			UNREACHABLE_VOID
 	}
 }
 
@@ -1189,13 +1220,12 @@ static Type *sema_expr_analyse_designator(SemaContext *context, Type *current, E
 	DesignatorElement **path = expr->designator_expr.path;
 
 	// Walk down into this path
-	bool is_constant = true;
 	bool did_report_error = false;
 	*member_ptr = NULL;
 	for (unsigned i = 0; i < vec_size(path); i++)
 	{
 		Decl *member_found;
-		Type *new_current = sema_find_type_of_element(context, current, &path, &i, &is_constant, &did_report_error, i == 0 ? max_index : NULL, &member_found);
+		Type *new_current = sema_find_type_of_element(context, current, &path, &i, &did_report_error, i == 0 ? max_index : NULL, &member_found);
 		if (!new_current)
 		{
 			if (!did_report_error) SEMA_ERROR(expr, "This is not a valid member of '%s'.", type_to_error_string(current));
@@ -1212,7 +1242,7 @@ INLINE bool sema_initializer_list_is_empty(Expr *value)
 	return expr_is_const_initializer(value) && value->const_expr.initializer->kind == CONST_INIT_ZERO;
 }
 
-static Type *sema_find_type_of_element(SemaContext *context, Type *type, DesignatorElement ***elements_ref, unsigned *curr_index, bool *is_constant, bool *did_report_error, ArrayIndex *max_index, Decl **member_ptr)
+static Type *sema_find_type_of_element(SemaContext *context, Type *type, DesignatorElement ***elements_ref, unsigned *curr_index, bool *did_report_error, ArrayIndex *max_index, Decl **member_ptr)
 {
 	Type *type_flattened = type_flatten(type);
 	DesignatorElement *element = (*elements_ref)[*curr_index];
@@ -1292,7 +1322,7 @@ static Type *sema_find_type_of_element(SemaContext *context, Type *type, Designa
 
 static ArrayIndex sema_analyse_designator_index(SemaContext *context, Expr *index)
 {
-	if (!sema_analyse_expr(context, index))
+	if (!sema_analyse_expr_rvalue(context, index))
 	{
 		return -1;
 	}
