@@ -3,6 +3,7 @@
 // a copy of which can be found in the LICENSE file.
 
 #include "sema_internal.h"
+#include "compiler_tests/benchmark.h"
 
 void parent_path(StringSlice *slice)
 {
@@ -116,6 +117,33 @@ void sema_analysis_pass_process_imports(Module *module)
 			import->import.module = import_module;
 NEXT:;
 		}
+		FOREACH_IDX(idx, Decl *, alias_module,  unit->module_aliases)
+		{
+			Path *path = alias_module->module_alias_decl.alias_path;
+			Module *import_module = global_context_find_module(path->module);
+
+			// 5. Do we find it?
+			if (!import_module)
+			{
+				PRINT_ERROR_AT(path, "No module named '%s' could be found, did you type the name right?", path->module);
+				continue;
+			}
+			alias_module->module_alias_decl.module = import_module;
+			alias_module->resolve_status = RESOLVE_DONE;
+			for (unsigned i = 0; i < idx; i++)
+			{
+				if (unit->module_aliases[i]->name == alias_module->name)
+				{
+					PRINT_ERROR_AT(alias_module, "The module alias must be unique.");
+					break;
+				}
+			}
+			if (alias_module->attributes)
+			{
+				PRINT_ERROR_AT(alias_module->attributes[0], "Module aliases cannot have attributes.");
+				break;
+			}
+		}
 		total_import_count += import_count;
 	}
 	(void)total_import_count; // workaround for clang 13.0
@@ -201,7 +229,7 @@ static bool exec_arg_append_to_scratch(Expr *arg)
 			scratch_buffer_append(arg->const_expr.global_ref->name);
 			return true;
 		case CONST_FAULT:
-			scratch_buffer_append(arg->const_expr.fault->name);
+			scratch_buffer_append(arg->const_expr.fault ? arg->const_expr.fault->name : "null");
 		return true;
 		case CONST_ENUM:
 			scratch_buffer_append(arg->const_expr.enum_val->name);
@@ -232,6 +260,7 @@ static bool exec_arg_append_to_scratch(Expr *arg)
 
 static Decl **sema_run_exec(CompilationUnit *unit, Decl *decl)
 {
+	double bench = bench_mark();
 	if (compiler.build.trust_level < TRUST_FULL)
 	{
 		RETURN_PRINT_ERROR_AT(NULL, decl, "'$exec' not permitted, trust level must be set to '--trust=full' to permit it.");
@@ -311,6 +340,7 @@ static Decl **sema_run_exec(CompilationUnit *unit, Decl *decl)
 	{
 		RETURN_PRINT_ERROR_AT(NULL, decl, "This $include would cause the maximum number of includes (%d) to be exceeded.", MAX_INCLUDE_DIRECTIVES);
 	}
+	compiler.exec_time += bench_mark() - bench;
 	return parse_include_file(file, unit);
 }
 
@@ -328,7 +358,7 @@ INLINE void register_includes(CompilationUnit *unit, Decl **decls)
 				include_decls = sema_load_include(unit, include);
 				break;
 			default:
-				UNREACHABLE
+				UNREACHABLE_VOID
 		}
 		FOREACH(Decl *, decl, include_decls)
 		{
@@ -464,13 +494,13 @@ CHECK_LINK:
 			unsigned args = vec_size(exprs);
 			ASSERT(args > 0 && "Should already have been checked.");
 			Expr *cond = args > 1 ? attr->exprs[0] : NULL;
-			if (cond && !sema_analyse_expr(&context, cond)) goto FAIL_CONTEXT;
+			if (cond && !sema_analyse_expr_rvalue(&context, cond)) goto FAIL_CONTEXT;
 			bool start = cond && expr_is_const_bool(cond) ? 1 : 0;
 			bool add = start == 0 ? true : cond->const_expr.b;
 			for (unsigned i = start; i < args; i++)
 			{
 				Expr *string = attr->exprs[i];
-				if (!sema_analyse_expr(&context, string)) goto FAIL_CONTEXT;
+				if (!sema_analyse_expr_rvalue(&context, string)) goto FAIL_CONTEXT;
 				if (!expr_is_const_string(string))
 				{
 					PRINT_ERROR_AT(string, "Expected a constant string here, usage is: "
@@ -595,10 +625,10 @@ INLINE void sema_analyse_inner_func_ptr(SemaContext *c, Decl *decl)
 	Type *inner;
 	switch (decl->decl_kind)
 	{
-		case DECL_DISTINCT:
+		case DECL_TYPEDEF:
 			inner = decl->distinct->type;
 			break;
-		case DECL_TYPEDEF:
+		case DECL_TYPE_ALIAS:
 			inner = decl->type->canonical;
 			break;
 		default:
@@ -680,40 +710,55 @@ void sema_analysis_pass_lambda(Module *module)
 	DEBUG_LOG("Pass finished with %d error(s).", compiler.context.errors_found);
 }
 
+static bool sema_check_interface(SemaContext *context, Decl *decl, TypeInfo *interface_type, TypeInfo *original_type)
+{
+	Decl *interface = interface_type->type->decl;
+	FOREACH(Decl *, method, interface->interface_methods)
+	{
+		Decl *matching_method = sema_decl_stack_resolve_symbol(method->name);
+		if (!matching_method)
+		{
+			if (method->func_decl.attr_optional) continue;
+			if (interface_type != original_type)
+			{
+				RETURN_SEMA_ERROR(original_type,
+				                  "'%s' was not fully implemented, the required method '%s' of interface '%s' needs to be implemented, did you forget it?",
+				                  original_type->type->decl->name, method->name, interface->name);
+			}
+			RETURN_SEMA_ERROR(original_type,
+			                  "'%s' was not fully implemented, the required method '%s' needs to be implemented, did you forget it?",
+			                  interface->name, method->name);
+		}
+		if (matching_method->decl_kind != DECL_FUNC)
+		{
+			if (method->func_decl.attr_optional) continue;
+			RETURN_SEMA_ERROR(matching_method, "'%s' was not fully implemented, it requires '%s' to be a function marked '@dynamic'.",
+					   interface->name, method->name);
+		}
+		if (!matching_method->func_decl.attr_dynamic)
+		{
+			SEMA_ERROR(matching_method, "'%s(...)' must be marked '@dynamic' as it matches the method '%s' in interface '%s'.",
+					   method->name, method->name, interface->name);
+			SEMA_NOTE(method, "Here is the interface method to implement.");
+			return false;
+		}
+	}
+	FOREACH(TypeInfo *, parent_interface, interface->interfaces)
+	{
+		if (!sema_check_interface(context, decl, parent_interface, original_type)) return false;
+	}
+	return true;
+}
 static inline bool sema_check_interfaces(SemaContext *context, Decl *decl)
 {
 	Decl **store = sema_decl_stack_store();
-	FOREACH(Decl *, method, decl->methods) sema_decl_stack_push(method);
+	sema_add_methods_to_decl_stack(context, decl);
 	FOREACH(TypeInfo *, interface_type, decl->interfaces)
 	{
-		Decl *interface = interface_type->type->decl;
-		FOREACH(Decl *, method, interface->interface_methods)
+		if (!sema_check_interface(context, decl, interface_type, interface_type))
 		{
-			Decl *matching_method = sema_decl_stack_resolve_symbol(method->name);
-			if (!matching_method)
-			{
-				if (method->func_decl.attr_optional) continue;
-				SEMA_ERROR(interface_type, "'%s' was not fully implemented, required method '%s' needs to be implemented, did you forget it?",
-				           interface->name, method->name);
-				sema_decl_stack_restore(store);
-				return false;
-			}
-			if (matching_method->decl_kind != DECL_FUNC)
-			{
-				if (method->func_decl.attr_optional) continue;
-				SEMA_ERROR(matching_method, "'%s' was not fully implemented, it requires '%s' to be a function marked '@dynamic'.",
-				           interface->name, method->name);
-				sema_decl_stack_restore(store);
-				return false;
-			}
-			if (!matching_method->func_decl.attr_dynamic)
-			{
-				SEMA_ERROR(matching_method, "'%s(...)' must be marked '@dynamic' as it matches the method '%s' in interface '%s'.",
-				           method->name, method->name, interface->name);
-				SEMA_NOTE(method, "Here is the interface method to implement.");
-				sema_decl_stack_restore(store);
-				return false;
-			}
+			sema_decl_stack_restore(store);
+			return false;
 		}
 	}
 	sema_decl_stack_restore(store);
@@ -732,7 +777,7 @@ void sema_analysis_pass_interface(Module *module)
 		{
 			switch (decl->decl_kind)
 			{
-				case DECL_DISTINCT:
+				case DECL_TYPEDEF:
 				case DECL_STRUCT:
 				case DECL_UNION:
 				case DECL_ENUM:
