@@ -40,11 +40,12 @@ const char *builtin_defines[NUMBER_OF_BUILTIN_DEFINES];
 const char *type_property_list[NUMBER_OF_TYPE_PROPERTIES];
 const char *kw_at_deprecated;
 const char *kw_at_ensure;
+const char *kw_at_enum_lookup;
+const char *kw_at_jump;
 const char *kw_at_param;
 const char *kw_at_pure;
 const char *kw_at_require;
 const char *kw_at_return;
-const char *kw_at_jump;
 const char *kw_in;
 const char *kw_inout;
 const char *kw_len;
@@ -62,6 +63,7 @@ const char *kw_self;
 const char *kw_std;
 const char *kw_std__core;
 const char *kw_std__core__types;
+const char *kw_std__core__runtime;
 const char *kw_std__io;
 const char *kw_type;
 const char *kw_typekind;
@@ -146,6 +148,7 @@ void symtab_init(uint32_t capacity)
 	kw_std = KW_DEF("std");
 	kw_std__core = KW_DEF("std::core");
 	kw_std__core__types = KW_DEF("std::core::types");
+	kw_std__core__runtime = KW_DEF("std::core::runtime");
 	kw_std__io = KW_DEF("std::io");
 	kw_type = KW_DEF("type");
 	kw_winmain = KW_DEF("wWinMain");
@@ -161,12 +164,15 @@ void symtab_init(uint32_t capacity)
 	type_property_list[TYPE_PROPERTY_ELEMENTS] = KW_DEF("elements");
 	type_property_list[TYPE_PROPERTY_EXTNAMEOF] = KW_DEF("extnameof");
 	type_property_list[TYPE_PROPERTY_FROM_ORDINAL] = KW_DEF("from_ordinal");
+
 	type_property_list[TYPE_PROPERTY_GET] = KW_DEF("get");
 	type_property_list[TYPE_PROPERTY_INF] = KW_DEF("inf");
 	type_property_list[TYPE_PROPERTY_INNER] = KW_DEF("inner");
 	type_property_list[TYPE_PROPERTY_IS_EQ] = KW_DEF("is_eq");
 	type_property_list[TYPE_PROPERTY_IS_ORDERED] = KW_DEF("is_ordered");
 	type_property_list[TYPE_PROPERTY_IS_SUBSTRUCT] = KW_DEF("is_substruct");
+	type_property_list[TYPE_PROPERTY_LOOKUP] = KW_DEF("lookup");
+	type_property_list[TYPE_PROPERTY_LOOKUP_FIELD] = KW_DEF("lookup_field");
 	type_property_list[TYPE_PROPERTY_KINDOF] = KW_DEF("kindof");
 	type_property_list[TYPE_PROPERTY_MEMBERSOF] = KW_DEF("membersof");
 	type_property_list[TYPE_PROPERTY_METHODSOF] = KW_DEF("methodsof");
@@ -178,6 +184,7 @@ void symtab_init(uint32_t capacity)
 	type_property_list[TYPE_PROPERTY_PARENTOF] = KW_DEF("parentof");
 	type_property_list[TYPE_PROPERTY_QNAMEOF] = KW_DEF("qnameof");
 	type_property_list[TYPE_PROPERTY_RETURNS] = KW_DEF("returns");
+	type_property_list[TYPE_PROPERTY_SET] = KW_DEF("set");
 	type_property_list[TYPE_PROPERTY_SIZEOF] = KW_DEF("sizeof");
 	type_property_list[TYPE_PROPERTY_TAGOF] = KW_DEF("tagof");
 	type_property_list[TYPE_PROPERTY_HAS_TAGOF] = KW_DEF("has_tagof");
@@ -217,6 +224,7 @@ void symtab_init(uint32_t capacity)
 	builtin_list[BUILTIN_EXP2] = KW_DEF("exp2");
 	builtin_list[BUILTIN_EXPECT] = KW_DEF("expect");
 	builtin_list[BUILTIN_EXPECT_WITH_PROBABILITY] = KW_DEF("expect_with_probability");
+	builtin_list[BUILTIN_FENCE] = KW_DEF("fence");
 	builtin_list[BUILTIN_FLOOR] = KW_DEF("floor");
 	builtin_list[BUILTIN_FMA] = KW_DEF("fma");
 	builtin_list[BUILTIN_FMULADD] = KW_DEF("fmuladd");
@@ -230,6 +238,8 @@ void symtab_init(uint32_t capacity)
 	builtin_list[BUILTIN_LOG10] = KW_DEF("log10");
 	builtin_list[BUILTIN_MASKED_LOAD] = KW_DEF("masked_load");
 	builtin_list[BUILTIN_MASKED_STORE] = KW_DEF("masked_store");
+	builtin_list[BUILTIN_MATRIX_MUL] = KW_DEF("matrix_mul");
+	builtin_list[BUILTIN_MATRIX_TRANSPOSE] = KW_DEF("matrix_transpose");
 	builtin_list[BUILTIN_MEMCOPY] = KW_DEF("memcpy");
 	builtin_list[BUILTIN_MEMCOPY_INLINE] = KW_DEF("memcpy_inline");
 	builtin_list[BUILTIN_MEMMOVE] = KW_DEF("memmove");
@@ -255,9 +265,11 @@ void symtab_init(uint32_t capacity)
 	builtin_list[BUILTIN_REVERSE] = KW_DEF("reverse");
 	builtin_list[BUILTIN_RETURNADDRESS] = KW_DEF("returnaddress");
 	builtin_list[BUILTIN_RINT] = KW_DEF("rint");
+	builtin_list[BUILTIN_RND] = KW_DEF("rnd");
 	builtin_list[BUILTIN_ROUND] = KW_DEF("round");
 	builtin_list[BUILTIN_ROUNDEVEN] = KW_DEF("roundeven");
 	builtin_list[BUILTIN_SAT_ADD] = KW_DEF("sat_add");
+	builtin_list[BUILTIN_SAT_MUL] = KW_DEF("sat_mul");
 	builtin_list[BUILTIN_SAT_SHL] = KW_DEF("sat_shl");
 	builtin_list[BUILTIN_SAT_SUB] = KW_DEF("sat_sub");
 	builtin_list[BUILTIN_SCATTER] = KW_DEF("scatter");
@@ -270,6 +282,7 @@ void symtab_init(uint32_t capacity)
 	builtin_list[BUILTIN_STR_FIND] = KW_DEF("str_find");
 	builtin_list[BUILTIN_SWIZZLE] = KW_DEF("swizzle");
 	builtin_list[BUILTIN_SWIZZLE2] = KW_DEF("swizzle2");
+	builtin_list[BUILTIN_SPRINTF] = KW_DEF("sprintf");
 	builtin_list[BUILTIN_SQRT] = KW_DEF("sqrt");
 	builtin_list[BUILTIN_SYSCALL] = KW_DEF("syscall");
 	builtin_list[BUILTIN_SYSCLOCK] = KW_DEF("sysclock");
@@ -308,14 +321,16 @@ void symtab_init(uint32_t capacity)
 
 	type = TOKEN_AT_IDENT;
 
-	kw_at_ensure = KW_DEF("@ensure");
 	kw_at_deprecated = KW_DEF("@deprecated");
+	kw_at_ensure = KW_DEF("@ensure");
+	kw_at_enum_lookup = KW_DEF("@enum_lookup_new");
+	kw_at_jump = KW_DEF("@jump");
 	kw_at_param = KW_DEF("@param");
 	kw_at_pure = KW_DEF("@pure");
 	kw_at_require = KW_DEF("@require");
 	kw_at_return = KW_DEF("@return");
-	kw_at_jump = KW_DEF("@jump");
 	attribute_list[ATTRIBUTE_ALIGN] = KW_DEF("@align");
+	attribute_list[ATTRIBUTE_ALLOW_DEPRECATED] = KW_DEF("@allow_deprecated");
 	attribute_list[ATTRIBUTE_BENCHMARK] = KW_DEF("@benchmark");
 	attribute_list[ATTRIBUTE_BIGENDIAN] = KW_DEF("@bigendian");
 	attribute_list[ATTRIBUTE_BUILTIN] = KW_DEF("@builtin");
@@ -331,6 +346,7 @@ void symtab_init(uint32_t capacity)
 	attribute_list[ATTRIBUTE_IF] = KW_DEF("@if");
 	attribute_list[ATTRIBUTE_INIT] = KW_DEF("@init");
 	attribute_list[ATTRIBUTE_INLINE] = KW_DEF("@inline");
+	attribute_list[ATTRIBUTE_JUMP] = kw_at_jump;
 	attribute_list[ATTRIBUTE_LINK] = KW_DEF("@link");
 	attribute_list[ATTRIBUTE_LITTLEENDIAN] = KW_DEF("@littleendian");
 	attribute_list[ATTRIBUTE_LOCAL] = KW_DEF("@local");
@@ -347,6 +363,8 @@ void symtab_init(uint32_t capacity)
 	attribute_list[ATTRIBUTE_NOSTRIP] = KW_DEF("@nostrip");
 	attribute_list[ATTRIBUTE_OBFUSCATE] = KW_DEF("@obfuscate");
 	attribute_list[ATTRIBUTE_OPERATOR] = KW_DEF("@operator");
+	attribute_list[ATTRIBUTE_OPERATOR_R] = KW_DEF("@operator_r");
+	attribute_list[ATTRIBUTE_OPERATOR_S] = KW_DEF("@operator_s");
 	attribute_list[ATTRIBUTE_OPTIONAL] = KW_DEF("@optional");
 	attribute_list[ATTRIBUTE_OVERLAP] = KW_DEF("@overlap");
 	attribute_list[ATTRIBUTE_PACKED] = KW_DEF("@packed");
@@ -354,8 +372,10 @@ void symtab_init(uint32_t capacity)
 	attribute_list[ATTRIBUTE_PURE] = kw_at_pure;
 	attribute_list[ATTRIBUTE_PUBLIC] = KW_DEF("@public");
 	attribute_list[ATTRIBUTE_REFLECT] = KW_DEF("@reflect");
+	attribute_list[ATTRIBUTE_SAFEINFER] = KW_DEF("@safeinfer");
 	attribute_list[ATTRIBUTE_SAFEMACRO] = KW_DEF("@safemacro");
 	attribute_list[ATTRIBUTE_SECTION] = KW_DEF("@section");
+	attribute_list[ATTRIBUTE_STRUCTLIKE] = KW_DEF("@structlike");
 	attribute_list[ATTRIBUTE_TEST] = KW_DEF("@test");
 	attribute_list[ATTRIBUTE_TAG] = KW_DEF("@tag");
 	attribute_list[ATTRIBUTE_UNUSED] = KW_DEF("@unused");
@@ -569,3 +589,65 @@ void *htable_get(HTable *table, void *key)
 	return NULL;
 }
 
+void pathtable_init(PathTable *table, uint32_t initial_size)
+{
+	ASSERT(initial_size && "Size must be larger than 0");
+	size_t size = next_highest_power_of_2(initial_size);
+
+	size_t mem_size = initial_size * sizeof(PathTableEntry);
+	table->entries = calloc_arena(mem_size);
+
+	table->mask = size - 1;
+}
+
+void pathtable_set(PathTable *table, Decl *value)
+{
+	ASSERT(value && "Cannot insert NULL");
+	ASSERT_SPAN(value, value->name);
+	const char *short_path = value->unit->module->short_path;
+	const char *name = value->name;
+	uint32_t idx = ((((uintptr_t)short_path) ^ ((uintptr_t)short_path) >> 8) ^(((uintptr_t)name) ^ ((uintptr_t)name) >> 8)) & table->mask;
+	PathTableEntry **entry_ref = &table->entries[idx];
+	PathTableEntry *entry = *entry_ref;
+	if (!entry)
+	{
+		entry = CALLOCS(HTEntry);
+		entry->short_path = short_path;
+		entry->name = name;
+		entry->value = value;
+		*entry_ref = entry;
+		return;
+	}
+	PathTableEntry *first_entry = entry;
+	do
+	{
+		if (entry->short_path == short_path && entry->name == name)
+		{
+			entry->value = poisoned_decl;
+			return;
+		}
+		entry = entry->next;
+	} while (entry);
+
+	entry = CALLOCS(HTEntry);
+	entry->short_path = short_path;
+	entry->name = name;
+	entry->value = value;
+	entry->next = first_entry;
+	*entry_ref = entry;
+}
+
+
+Decl *pathtable_get(PathTable *table, const char *short_path, const char *name)
+{
+	uint32_t idx = ((((uintptr_t)short_path) ^ ((uintptr_t)short_path) >> 8) ^(((uintptr_t)name) ^ ((uintptr_t)name) >> 8)) & table->mask;
+	PathTableEntry *entry = table->entries[idx];
+	if (!entry) return NULL;
+	do
+	{
+		if (entry->short_path == short_path && entry->name == name) return entry->value;
+		entry = entry->next;
+	} while (entry);
+	return NULL;
+}
+
