@@ -120,7 +120,7 @@ static bool os_target_use_thread_local(OsType os)
 		case OS_TYPE_NONE:
 			return false;
 		case OS_DARWIN_TYPES:
-		case OS_TYPE_FREE_BSD:
+		case OS_TYPE_FREEBSD:
 		case OS_TYPE_LINUX:
 		case OS_TYPE_ANDROID:
 		case OS_TYPE_NETBSD:
@@ -251,7 +251,7 @@ static inline void target_setup_arm_abi(void)
 			compiler.platform.arm.abi_variant = ARM_ABI_AAPCS;
 			break;
 	}
-	UNREACHABLE
+	UNREACHABLE_VOID
 }
 
 static inline void target_setup_x86_abi(BuildTarget *target)
@@ -269,7 +269,7 @@ static inline void target_setup_x86_abi(BuildTarget *target)
 	{
 		case OS_DARWIN_TYPES:
 		case OS_TYPE_DRAGON_FLY:
-		case OS_TYPE_FREE_BSD:
+		case OS_TYPE_FREEBSD:
 		case OS_TYPE_ELFIAMCU:
 		case OS_TYPE_OPENBSD:
 		case OS_TYPE_WIN32:
@@ -731,7 +731,7 @@ static void x86_features_add_feature(X86Features *cpu_features, X86Feature featu
 			x86_features_add_feature(cpu_features, X86_FEAT_AVX512FP16);
 			return;
 	}
-	UNREACHABLE
+	UNREACHABLE_VOID
 }
 
 
@@ -961,7 +961,7 @@ static void x86features_from_cpu(X86Features *cpu_features, X86CpuSet cpu_set)
 			x86_features_from_host(cpu_features);
 			return;
 	}
-	UNREACHABLE
+	UNREACHABLE_VOID
 }
 
 static inline bool x86_has_all_features(X86Features *feature_to_test, X86Features *features_to_match)
@@ -1279,7 +1279,7 @@ static OsType os_from_llvm_string(StringSlice os_string)
 	STRCASE("cloudabi", OS_TYPE_CLOUD_ABI)
 	STRCASE("darwin", OS_TYPE_MACOSX)
 	STRCASE("dragonfly", OS_TYPE_DRAGON_FLY)
-	STRCASE("freebsd", OS_TYPE_FREE_BSD)
+	STRCASE("freebsd", OS_TYPE_FREEBSD)
 	STRCASE("fuchsia", OS_TYPE_FUCHSIA)
 	STRCASE("ios", OS_TYPE_IOS)
 	STRCASE("kfreebsd", OS_TYPE_KFREEBSD)
@@ -1480,7 +1480,7 @@ static ObjectFormatType object_format_from_os(OsType os, ArchType arch_type)
 		case OS_TYPE_ANDROID:
 		case OS_TYPE_NETBSD:
 		case OS_TYPE_OPENBSD:
-		case OS_TYPE_FREE_BSD:
+		case OS_TYPE_FREEBSD:
 			return OBJ_FORMAT_ELF;
 		case OS_DARWIN_TYPES:
 			return OBJ_FORMAT_MACHO;
@@ -1535,7 +1535,7 @@ static unsigned os_target_c_type_bits(OsType os, ArchType arch, CType type)
 		case OS_TYPE_LINUX:
 		case OS_TYPE_ANDROID:
 		case OS_TYPE_NONE:
-		case OS_TYPE_FREE_BSD:
+		case OS_TYPE_FREEBSD:
 		case OS_TYPE_NETBSD:
 		case OS_TYPE_OPENBSD:
 		case OS_TYPE_WASI:
@@ -1578,7 +1578,8 @@ static AlignData os_target_alignment_of_int(OsType os, ArchType arch, uint32_t b
 	{
 		case ARCH_TYPE_UNKNOWN:
 		case ARCH_UNSUPPORTED:
-			UNREACHABLE
+			UNREACHABLE_VOID;
+			return (AlignData) { 0, 0 };
 		case ARCH_TYPE_ARM:
 		case ARCH_TYPE_THUMB:
 		case ARCH_TYPE_ARMB:
@@ -1614,7 +1615,8 @@ static AlignData os_target_alignment_of_int(OsType os, ArchType arch, uint32_t b
 			if (os == OS_TYPE_WIN32 || os == OS_TYPE_NACL) return (AlignData) { 64, 64 };
 			return (AlignData) { 32, 64 };
 	}
-	UNREACHABLE
+	UNREACHABLE_VOID;
+	return (AlignData) { 0, 0 };
 }
 
 static unsigned arch_big_endian(ArchType arch)
@@ -1653,7 +1655,8 @@ static AlignData os_target_alignment_of_float(OsType os, ArchType arch, uint32_t
 	{
 		case ARCH_TYPE_UNKNOWN:
 		case ARCH_UNSUPPORTED:
-			UNREACHABLE
+			UNREACHABLE_VOID;
+			break;
 		case ARCH_TYPE_X86:
 			if (os == OS_TYPE_ELFIAMCU && bits >= 32) return (AlignData) { 32, 32 };
 			if (os == OS_TYPE_WIN32 || os == OS_TYPE_NACL)
@@ -1685,7 +1688,8 @@ static AlignData os_target_alignment_of_float(OsType os, ArchType arch, uint32_t
 			if (bits == 128 && os == OS_TYPE_ELFIAMCU) return (AlignData) { 32, 32 };
 			return (AlignData) { bits , bits };
 	}
-	UNREACHABLE
+	UNREACHABLE_VOID;
+	return (AlignData) { 0, 0 };
 }
 
 static const char *os_dynamic_library_suffix(OsType os)
@@ -1715,7 +1719,7 @@ static RelocModel arch_os_reloc_default(ArchType arch, OsType os, EnvironmentTyp
 				return RELOC_BIG_PIC;
 			case OS_TYPE_WASI:
 				return RELOC_NONE;
-			case OS_TYPE_FREE_BSD:
+			case OS_TYPE_FREEBSD:
 			case OS_TYPE_NETBSD:
 			case OS_TYPE_UNKNOWN:
 			case OS_TYPE_NONE:
@@ -1734,7 +1738,7 @@ static RelocModel arch_os_reloc_default(ArchType arch, OsType os, EnvironmentTyp
 	{
 		case OS_TYPE_UNKNOWN:
 		case OS_TYPE_NONE:
-		case OS_TYPE_FREE_BSD:
+		case OS_TYPE_FREEBSD:
 		case OS_TYPE_NETBSD:
 			switch (arch)
 			{
@@ -1773,7 +1777,7 @@ static bool arch_os_pic_default_forced(ArchType arch, OsType os)
 		case OS_TYPE_WASI:
 		case OS_TYPE_UNKNOWN:
 		case OS_TYPE_NONE:
-		case OS_TYPE_FREE_BSD:
+		case OS_TYPE_FREEBSD:
 		case OS_TYPE_LINUX:
 		case OS_TYPE_ANDROID:
 		case OS_TYPE_NETBSD:
@@ -1834,13 +1838,25 @@ void *llvm_target_machine_create(void)
 	{
 		llvm_initialized = true;
 #if XTENSA_AVAILABLE
+#ifndef XTENSA_DISABLE
 		INITIALIZE_TARGET(Xtensa);
 #endif
+#endif
+#ifndef ARM_DISABLE
 		INITIALIZE_TARGET(ARM);
+#endif
+#ifndef AARCH64_DISABLE
 		INITIALIZE_TARGET(AArch64);
+#endif
+#ifndef RISCV_DISABLE
 		INITIALIZE_TARGET(RISCV);
+#endif
+#ifndef WASM_DISABLE
 		INITIALIZE_TARGET(WebAssembly);
+#endif
+#ifndef X86_DISABLE
 		INITIALIZE_TARGET(X86);
+#endif
 		// To support more targets, add them above.
 	}
 	char *err = NULL;
@@ -1912,7 +1928,7 @@ void target_setup(BuildTarget *target)
 	switch (target->optlevel)
 	{
 		case OPTIMIZATION_NOT_SET:
-			UNREACHABLE;
+			UNREACHABLE_VOID;
 		case OPTIMIZATION_AGGRESSIVE:
 			level = LLVMCodeGenLevelAggressive;
 			break;
@@ -1926,7 +1942,7 @@ void target_setup(BuildTarget *target)
 			level = LLVMCodeGenLevelNone;
 			break;
 		default:
-			UNREACHABLE;
+			UNREACHABLE_VOID;
 	}
 
 	compiler.platform.llvm_opt_level = (int)level;
@@ -1944,11 +1960,12 @@ void target_setup(BuildTarget *target)
 	compiler.platform.arch = arch_from_llvm_string(slice_next_token(&target_triple_string, '-'));
 	if (!arch_is_supported(compiler.platform.arch))
 	{
-		printf("WARNING! This architecture is unsupported.\n");
+		OUTF("WARNING! This architecture is unsupported.\n");
 	}
 	compiler.platform.vendor = vendor_from_llvm_string(slice_next_token(&target_triple_string, '-'));
 	compiler.platform.os = os_from_llvm_string(slice_next_token(&target_triple_string, '-'));
 	compiler.platform.environment_type = environment_type_from_llvm_string(target_triple_string);
+	if (compiler.platform.environment_type == ENV_TYPE_ANDROID) compiler.platform.os = OS_TYPE_ANDROID;
 
 	if (target->debug_info == DEBUG_INFO_NOT_SET)
 	{
@@ -2000,7 +2017,7 @@ void target_setup(BuildTarget *target)
 	switch (compiler.platform.arch)
 	{
 		case ARCH_UNSUPPORTED:
-			UNREACHABLE
+			UNREACHABLE_VOID
 			break;
 		case ARCH_TYPE_AARCH64:
 		case ARCH_TYPE_AARCH64_BE:
